@@ -17,6 +17,17 @@ import scipy.sparse as sp
 import GridCal.Engine.Core.topology as tp
 
 
+def get_bus_indices(C_branch_bus):
+    F = np.zeros(C_branch_bus.shape[0], dtype=int)
+
+    for j in range(C_branch_bus.shape[1]):
+        for l in range(C_branch_bus.indptr[j], C_branch_bus.indptr[j + 1]):
+            i = C_branch_bus.indices[l]  # row index
+            F[i] = j
+
+    return F
+
+
 class BranchData:
 
     def __init__(self, nbr, nbus, ntime=1):
@@ -30,8 +41,11 @@ class BranchData:
 
         self.branch_names = np.empty(self.nbr, dtype=object)
 
+        self.branch_dc = np.zeros(self.nbr, dtype=int)
+
         self.branch_active = np.zeros((nbr, ntime), dtype=int)
         self.branch_rates = np.zeros((nbr, ntime), dtype=float)
+        self.branch_contingency_rates = np.zeros((nbr, ntime), dtype=float)
 
         self.F = np.zeros(self.nbr, dtype=int)  # indices of the "from" buses
         self.T = np.zeros(self.nbr, dtype=int)  # indices of the "to" buses
@@ -72,6 +86,9 @@ class BranchData:
         self.alpha3 = np.zeros(self.nbr)  # converter losses parameter (alpha3)
         self.control_mode = np.zeros(self.nbr, dtype=object)
 
+        self.contingency_enabled = np.ones(self.nbr, dtype=int)
+        self.monitor_loading = np.ones(self.nbr, dtype=int)
+
         self.C_branch_bus_f = sp.lil_matrix((self.nbr, nbus), dtype=int)  # connectivity branch with their "from" bus
         self.C_branch_bus_t = sp.lil_matrix((self.nbr, nbus), dtype=int)  # connectivity branch with their "to" bus
 
@@ -92,8 +109,8 @@ class BranchData:
         data = BranchData(nbr=len(elm_idx), nbus=len(bus_idx))
 
         data.branch_names = self.branch_names[elm_idx]
-        data.F = self.F[elm_idx]
-        data.T = self.T[elm_idx]
+        # data.F = self.F[elm_idx]
+        # data.T = self.T[elm_idx]
         data.R = self.R[elm_idx]
         data.X = self.X[elm_idx]
         data.G = self.G[elm_idx]
@@ -109,9 +126,10 @@ class BranchData:
         data.alpha3 = self.alpha3[elm_idx]
 
         data.control_mode = self.control_mode[elm_idx]
-
         data.branch_active = self.branch_active[tidx]
+        data.branch_dc = self.branch_dc[tidx]
         data.branch_rates = self.branch_rates[tidx]
+        data.branch_contingency_rates = self.branch_contingency_rates[tidx]
         data.m = self.m[tidx]
         data.m_min = self.m_min[elm_idx]
         data.m_max = self.m_max[elm_idx]
@@ -126,9 +144,15 @@ class BranchData:
         data.vf_set = self.vf_set[tidx]
         data.vt_set = self.vt_set[tidx]
 
+        data.contingency_enabled = self.contingency_enabled[tidx]
+        data.monitor_loading = self.monitor_loading[tidx]
+
         data.C_branch_bus_f = self.C_branch_bus_f[np.ix_(elm_idx, bus_idx)]
         data.C_branch_bus_t = self.C_branch_bus_t[np.ix_(elm_idx, bus_idx)]
 
+        data.F = get_bus_indices(data.C_branch_bus_f)
+        data.T = get_bus_indices(data.C_branch_bus_t)
+
         return data
 
     def get_island(self, bus_idx):
@@ -139,6 +163,16 @@ class BranchData:
         """
         return tp.get_elements_of_the_island(self.C_branch_bus_f + self.C_branch_bus_t, bus_idx)
 
+    def get_contingency_enabled_indices(self):
+
+        return np.where(self.contingency_enabled == 1)[0]
+
+    def get_dc_indices(self):
+        return np.where(self.branch_dc != 0)[0]
+
+    def get_ac_indices(self):
+        return np.where(self.branch_dc == 0)[0]
+
     def __len__(self):
         return self.nbr
 
@@ -170,35 +204,8 @@ class BranchOpfData(BranchData):
         else:
             tidx = np.ix_(elm_idx, time_idx)
 
-        data = BranchOpfData(nbr=len(elm_idx), nbus=len(bus_idx))
-
-        data.branch_names = self.branch_names[elm_idx]
-        data.F = self.F[elm_idx]
-        data.T = self.T[elm_idx]
-        data.R = self.R[elm_idx]
-        data.X = self.X[elm_idx]
-        data.G = self.G[elm_idx]
-        data.B = self.B[elm_idx]
-        data.k = self.k[elm_idx]
-        data.tap_t = self.tap_f[elm_idx]
-        data.tap_f = self.tap_t[elm_idx]
-        data.Kdp = self.Kdp[elm_idx]
-        data.control_mode = self.control_mode[elm_idx]
-
-        data.branch_active = self.branch_active[tidx]
-        data.branch_rates = self.branch_rates[tidx]
-        data.m = self.m[tidx]
-        data.theta = self.theta[tidx]
-        data.Beq = self.Beq[tidx]
-        data.G0 = self.G0[tidx]
-        data.Pfset = self.Pfset[tidx]
-        data.Qfset = self.Qfset[tidx]
-        data.vf_set = self.vf_set[tidx]
-        data.vt_set = self.vt_set[tidx]
+        data = super().slice(elm_idx, bus_idx, time_idx)
 
         data.branch_cost = self.branch_cost[tidx]
 
-        data.C_branch_bus_f = self.C_branch_bus_f[np.ix_(elm_idx, bus_idx)]
-        data.C_branch_bus_t = self.C_branch_bus_t[np.ix_(elm_idx, bus_idx)]
-
         return data
