@@ -15,20 +15,27 @@ def get_bus_data(circuit: MultiCircuit, time_series=False, ntime=1):
     """
     bus_data = BusData(nbus=len(circuit.buses), ntime=ntime)
 
+    areas_dict = {elm: k for k, elm in enumerate(circuit.areas)}
+
     for i, bus in enumerate(circuit.buses):
 
         # bus parameters
         bus_data.bus_names[i] = bus.name
         bus_data.Vmin[i] = bus.Vmin
         bus_data.Vmax[i] = bus.Vmax
+        bus_data.bus_types[i] = bus.determine_bus_type().value
+
+        if bus.area in areas_dict.keys():
+            bus_data.areas[i] = areas_dict[bus.area]
+        else:
+            bus_data.areas[i] = 0
 
         if time_series:
             bus_data.bus_active[i, :] = bus.active_prof
+            bus_data.bus_types_prof[i, :] = bus.determine_bus_type_prof()
         else:
             bus_data.bus_active[i] = bus.active
 
-        bus_data.bus_types[i] = bus.determine_bus_type().value
-
     return bus_data
 
 
@@ -209,7 +216,7 @@ def get_generator_data(circuit: MultiCircuit, bus_dict, Vbus, logger: Logger,
                 data.generator_cost[k] = elm.Cost
 
             if opf_results is not None:
-                data.generator_p[k] = opf_results.generators_power[k] - opf_results.generation_shedding[k]
+                data.generator_p[k] = opf_results.generator_power[k] - opf_results.generator_shedding[k]
 
         data.C_bus_gen[i, k] = 1
 
@@ -547,6 +554,7 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
         if time_series:
             data.branch_active[i, :] = elm.active_prof
             data.branch_rates[i, :] = elm.rate_prof
+            data.branch_contingency_rates[i, :] = elm.rate_prof * elm.contingency_factor
 
             if opf:
                 data.branch_cost[i, :] = elm.Cost_prof
@@ -554,6 +562,7 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
         else:
             data.branch_active[i] = elm.active
             data.branch_rates[i] = elm.rate
+            data.branch_contingency_rates[i] = elm.rate * elm.contingency_factor
 
             if opf:
                 data.branch_cost[i] = elm.Cost
@@ -578,6 +587,9 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
         data.X[i] = elm.X
         data.B[i] = elm.B
 
+        data.contingency_enabled[i] = int(elm.contingency_enabled)
+        data.monitor_loading[i] = int(elm.monitor_loading)
+
     # 2-winding transformers
     for i, elm in enumerate(circuit.transformers2w):
         ii = i + nline
@@ -591,12 +603,14 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
         if time_series:
             data.branch_active[ii, :] = elm.active_prof
             data.branch_rates[ii, :] = elm.rate_prof
+            data.branch_contingency_rates[ii, :] = elm.rate_prof * elm.contingency_factor
 
             if opf:
                 data.branch_cost[ii, :] = elm.Cost_prof
         else:
             data.branch_active[ii] = elm.active
             data.branch_rates[ii] = elm.rate
+            data.branch_contingency_rates[ii] = elm.rate * elm.contingency_factor
 
             if opf:
                 data.branch_cost[ii, :] = elm.Cost
@@ -621,6 +635,9 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
         data.control_mode[ii] = elm.control_mode
         data.tap_f[ii], data.tap_t[ii] = elm.get_virtual_taps()
 
+        data.contingency_enabled[ii] = int(elm.contingency_enabled)
+        data.monitor_loading[ii] = int(elm.monitor_loading)
+
         if elm.control_mode == TransformerControlType.Vt:
             Vbus[t] = elm.vset
 
@@ -640,11 +657,15 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
         if time_series:
             data.branch_active[ii, :] = elm.active_prof
             data.branch_rates[ii, :] = elm.rate_prof
+            data.branch_contingency_rates[ii, :] = elm.rate_prof * elm.contingency_factor
+
             if opf:
                 data.branch_cost[ii, :] = elm.Cost_prof
         else:
             data.branch_active[ii] = elm.active
             data.branch_rates[ii] = elm.rate
+            data.branch_contingency_rates[ii] = elm.rate * elm.contingency_factor
+
             if opf:
                 data.branch_cost[ii] = elm.Cost
 
@@ -673,6 +694,8 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
         data.vf_set[ii] = elm.Vac_set
         data.vt_set[ii] = elm.Vdc_set
         data.control_mode[ii] = elm.control_mode
+        data.contingency_enabled[ii] = int(elm.contingency_enabled)
+        data.monitor_loading[ii] = int(elm.monitor_loading)
 
         '''
         type_0_free = '0:Free'
@@ -710,15 +733,20 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
         t = bus_dict[elm.bus_to]
 
         data.branch_names[ii] = elm.name
+        data.branch_dc[ii] = 1
 
         if time_series:
             data.branch_active[ii, :] = elm.active_prof
             data.branch_rates[ii, :] = elm.rate_prof
+            data.branch_contingency_rates[ii, :] = elm.rate_prof * elm.contingency_factor
+
             if opf:
                 data.branch_cost[ii, :] = elm.Cost_prof
         else:
             data.branch_active[ii] = elm.active
             data.branch_rates[ii] = elm.rate
+            data.branch_contingency_rates[ii] = elm.rate * elm.contingency_factor
+
             if opf:
                 data.branch_cost[ii] = elm.Cost
 
@@ -727,6 +755,9 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
         data.F[ii] = f
         data.T[ii] = t
 
+        data.contingency_enabled[ii] = int(elm.contingency_enabled)
+        data.monitor_loading[ii] = int(elm.monitor_loading)
+
         if apply_temperature:
             data.R[ii] = elm.R_corrected
         else:
@@ -750,11 +781,15 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
         if time_series:
             data.branch_active[ii, :] = elm.active_prof
             data.branch_rates[ii, :] = elm.rate_prof
+            data.branch_contingency_rates[ii, :] = elm.rate_prof * elm.contingency_factor
+
             if opf:
                 data.branch_cost[ii, :] = elm.Cost_prof
         else:
             data.branch_active[ii] = elm.active
             data.branch_rates[ii] = elm.rate
+            data.branch_contingency_rates[ii] = elm.rate * elm.contingency_factor
+
             if opf:
                 data.branch_cost[ii] = elm.Cost
 
@@ -769,6 +804,9 @@ def get_branch_data(circuit: MultiCircuit, bus_dict, Vbus, apply_temperature,
 
         data.Pfset[ii] = elm.Pfset
 
+        data.contingency_enabled[ii] = int(elm.contingency_enabled)
+        data.monitor_loading[ii] = int(elm.monitor_loading)
+
     return data
 
 
@@ -806,6 +844,9 @@ def get_hvdc_data(circuit: MultiCircuit, bus_dict, bus_types, time_series=False,
             data.Vset_t[i] = elm.Vset_t
 
         data.loss_factor[i] = elm.loss_factor
+        data.r[i] = elm.r
+        data.control_mode[i] = elm.control_mode
+
         data.Qmin_f[i] = elm.Qmin_f
         data.Qmax_f[i] = elm.Qmax_f
         data.Qmin_t[i] = elm.Qmin_t
