@@ -212,9 +212,6 @@ class MultiCircuit:
                                       Area(),
                                       Country()]
 
-        # this dictionary is set by the GUI and it is not meant to be used by the API operations
-        self.results_dictionary = dict()
-
         # dictionary of profile magnitudes per object
         self.profile_magnitudes = dict()
 
@@ -920,7 +917,7 @@ class MultiCircuit:
 
         self.buses.append(obj)
 
-    def delete_bus(self, obj: Bus):
+    def delete_bus(self, obj: Bus, ask=True):
         """
         Delete a :ref:`Bus<bus>` object from the grid.
 
@@ -933,9 +930,10 @@ class MultiCircuit:
         for branch_list in self.get_branch_lists():
             for i in range(len(branch_list) - 1, -1, -1):
                 if branch_list[i].bus_from == obj or branch_list[i].bus_to == obj:
-                    deleted_elm = branch_list.pop(i)
-                    if deleted_elm.graphic_obj is not None:
-                        deleted_elm.graphic_obj.remove()
+                    if branch_list[i].graphic_obj is not None:
+                        branch_list[i].graphic_obj.remove(ask=ask)
+                    else:
+                        self.delete_branch(branch_list[i])
 
         # remove the bus itself
         if obj in self.buses:
@@ -1411,6 +1409,104 @@ class MultiCircuit:
         """
         self.countries.pop(i)
 
+    def convert_line_to_hvdc(self, line: Line) -> HvdcLine:
+        """
+        Convert a line to HVDC, this is the GUI way to create HVDC objects
+        :param line: Line instance
+        :return: HvdcLine
+        """
+        hvdc = HvdcLine(bus_from=line.bus_from,
+                        bus_to=line.bus_to,
+                        name='HVDC Line',
+                        active=line.active,
+                        rate=line.rate,
+                        active_prof=line.active_prof,
+                        rate_prof=line.rate_prof)
+
+        # add device to the circuit
+        self.add_hvdc(hvdc)
+
+        # delete the line from the circuit
+        self.delete_line(line)
+
+        return hvdc
+
+    def convert_line_to_transformer(self, line: Line) -> Transformer2W:
+        """
+        Convert a line to Transformer
+        :param line: Line instance
+        :return: Transformer2W
+        """
+        transformer = Transformer2W(bus_from=line.bus_from,
+                                    bus_to=line.bus_to,
+                                    name='Transformer',
+                                    active=line.active,
+                                    rate=line.rate,
+                                    r=line.R,
+                                    x=line.X,
+                                    b=line.B,
+                                    active_prof=line.active_prof,
+                                    rate_prof=line.rate_prof)
+
+        # add device to the circuit
+        self.add_transformer2w(transformer)
+
+        # delete the line from the circuit
+        self.delete_line(line)
+
+        return transformer
+
+    def convert_line_to_vsc(self, line: Line) -> VSC:
+        """
+        Convert a line to voltage source converter
+        :param line: Line instance
+        :return: Nothing
+        """
+        vsc = VSC(bus_from=line.bus_from,
+                  bus_to=line.bus_to,
+                  name='VSC',
+                  active=line.active,
+                  rate=line.rate,
+                  r1=line.R,
+                  x1=line.X,
+                  Beq=line.B,
+                  m=1.0,
+                  active_prof=line.active_prof,
+                  rate_prof=line.rate_prof)
+
+        # add device to the circuit
+        self.add_vsc(vsc)
+
+        # delete the line from the circuit
+        self.delete_line(line)
+
+        return vsc
+
+    def convert_line_to_upfc(self, line: Line) -> UPFC:
+        """
+        Convert a line to voltage source converter
+        :param line: Line instance
+        :return: Nothing
+        """
+        upfc = UPFC(bus_from=line.bus_from,
+                    bus_to=line.bus_to,
+                    name='UPFC',
+                    active=line.active,
+                    rate=line.rate,
+                    rl=line.R,
+                    xl=line.X,
+                    bl=line.B,
+                    active_prof=line.active_prof,
+                    rate_prof=line.rate_prof)
+
+        # add device to the circuit
+        self.add_upfc(upfc)
+
+        # delete the line from the circuit
+        self.delete_line(line)
+
+        return upfc
+
     def plot_graph(self, ax=None):
         """
         Plot the grid.
@@ -1991,3 +2087,108 @@ class MultiCircuit:
                 logger.add_error("Missing in the model", col_name)
 
         return logger
+
+    def get_bus_area_indices(self):
+        """
+        Get array of area indices for each bus
+        :return:
+        """
+        d = {elm: k for k, elm in enumerate(self.areas)}
+
+        lst = np.zeros(len(self.buses), dtype=int)
+        for k, bus in enumerate(self.buses):
+            lst[k] = d[bus.area]
+        return lst
+
+    def get_area_buses(self, area: Area) -> List[Tuple[int, Bus]]:
+        """
+        Get the selected buses
+        :return:
+        """
+        lst: List[Tuple[int, Bus]] = list()
+        for k, bus in enumerate(self.buses):
+            if bus.area == area:
+                lst.append((k, bus))
+        return lst
+
+    def get_areas_buses(self, areas: List[Area]) -> List[Tuple[int, Bus]]:
+        """
+        Get the selected buses
+        :return:
+        """
+        lst: List[Tuple[int, Bus]] = list()
+        for k, bus in enumerate(self.buses):
+            if bus.area in areas:
+                lst.append((k, bus))
+        return lst
+
+    def get_zone_buses(self, zone: Zone) -> List[Tuple[int, Bus]]:
+        """
+        Get the selected buses
+        :return:
+        """
+        lst: List[Tuple[int, Bus]] = list()
+        for k, bus in enumerate(self.buses):
+            if bus.zone == zone:
+                lst.append((k, bus))
+        return lst
+
+    def get_inter_area_branches(self, a1: Area, a2: Area):
+        """
+        Get the inter-area branches
+        :param a1: Area from
+        :param a2: Area to
+        :return: List of (branch index, branch object, flow sense w.r.t the area exchange)
+        """
+        lst: List[Tuple[int, object, float]] = list()
+        for k, branch in enumerate(self.get_branches()):
+            if branch.bus_from.area == a1 and branch.bus_to.area == a2:
+                lst.append((k, branch, 1.0))
+            elif branch.bus_from.area == a2 and branch.bus_to.area == a1:
+                lst.append((k, branch, -1.0))
+        return lst
+
+    def get_inter_areas_branches(self, a1: List[Area], a2: List[Area]):
+        """
+        Get the inter-area branches. HVDC branches are not considered
+        :param a1: Area from
+        :param a2: Area to
+        :return: List of (branch index, branch object, flow sense w.r.t the area exchange)
+        """
+        lst: List[Tuple[int, object, float]] = list()
+        for k, branch in enumerate(self.get_branches_wo_hvdc()):
+            if branch.bus_from.area in a1 and branch.bus_to.area in a2:
+                lst.append((k, branch, 1.0))
+            elif branch.bus_from.area in a2 and branch.bus_to.area in a1:
+                lst.append((k, branch, -1.0))
+        return lst
+
+    def get_inter_areas_hvdc_branches(self, a1: List[Area], a2: List[Area]):
+        """
+        Get the inter-area branches
+        :param a1: Area from
+        :param a2: Area to
+        :return: List of (branch index, branch object, flow sense w.r.t the area exchange)
+        """
+        lst: List[Tuple[int, object, float]] = list()
+        for k, branch in enumerate(self.hvdc_lines):
+            if branch.bus_from.area in a1 and branch.bus_to.area in a2:
+                lst.append((k, branch, 1.0))
+            elif branch.bus_from.area in a2 and branch.bus_to.area in a1:
+                lst.append((k, branch, -1.0))
+        return lst
+
+    def get_inter_zone_branches(self, z1: Zone, z2: Zone):
+        """
+        Get the inter-area branches
+        :param z1: Zone from
+        :param z2: Zone to
+        :return: List of (branch index, branch object, flow sense w.r.t the area exchange)
+        """
+        lst: List[Tuple[int, object, float]] = list()
+        for k, branch in enumerate(self.get_branches()):
+            if branch.bus_from.zone == z1 and branch.bus_to.zone == z2:
+                lst.append((k, branch, 1.0))
+            elif branch.bus_from.zone == z2 and branch.bus_to.zone == z1:
+                lst.append((k, branch, -1.0))
+        return lst
\ No newline at end of file
