@@ -14,7 +14,9 @@
 # along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
 
 import numpy as np
+import numba as nb
 import pandas as pd
+import scipy.sparse as sp
 from typing import List
 
 from GridCal.Engine.basic_structures import Logger
@@ -31,6 +33,93 @@ from GridCal.Engine.Devices.enumerations import TransformerControlType, Converte
 sparse_type = get_sparse_type()
 
 
+@nb.njit()
+def compose_generator_voltage_profile(nbus, ntime,
+                                      gen_bus_indices, gen_vset, gen_status, gen_is_controlled,
+                                      bat_bus_indices, bat_vset, bat_status, bat_is_controlled,
+                                      hvdc_bus_f, hvdc_bus_t, hvdc_status, hvdc_vf, hvdc_vt,
+                                      iBeqv, iVtma, VfBeqbus, Vtmabus, branch_status, br_vf, br_vt):
+    """
+    Get the array of voltage set points per bus
+    :param nbus: number of buses
+    :param ntime: number of time steps
+    :param gen_bus_indices: array of bus indices per generator
+    :param gen_vset: array of voltage set points (ngen, ntime)
+    :param gen_status: array of generator status (ngen, ntime)
+    :param gen_is_controlled: array of values indicating if a generator controls the voltage or not (ngen)
+    :param bat_bus_indices:  array of bus indices per battery
+    :param bat_vset: array of voltage set points (nbat, ntime)
+    :param bat_status: array of battery status (nbat, ntime)
+    :param bat_is_controlled: array of values indicating if a battery controls the voltage or not (ngen)
+    :param hvdc_bus_f:
+    :param hvdc_bus_t:
+    :param hvdc_status:
+    :param hvdc_vf
+    :param hvdc_vt
+    :param iBeqv: indices of the branches when controlling Vf with Beq
+    :param iVtma: indices of the branches when controlling Vt with ma
+    :param VfBeqbus: indices of the buses where Vf is controlled by Beq
+    :param Vtmabus: indices of the buses where Vt is controlled by ma
+    :param branch_status:
+    :param br_vf:
+    :param br_vt:
+    :return: Voltage set points array per bus (nbus, ntime)
+    """
+    V = np.ones((nbus, ntime), dtype=nb.complex128)
+    used = np.zeros((nbus, ntime), dtype=nb.int8)
+
+    # generators
+    for i, bus_idx in enumerate(gen_bus_indices):
+        if gen_is_controlled[i]:
+            for t in range(ntime):
+                if used[bus_idx, t] == 0:
+                    if gen_status[i, t]:
+                        V[bus_idx, t] = complex(gen_vset[i, t], 0)
+                        used[bus_idx, t] = 1
+
+    # batteries
+    for i, bus_idx in enumerate(bat_bus_indices):
+        if bat_is_controlled[i]:
+            for t in range(ntime):
+                if used[bus_idx, t] == 0:
+                    if bat_status[i, t]:
+                        V[bus_idx, t] = complex(bat_vset[i, t], 0)
+                        used[bus_idx, t] = 1
+
+    # HVDC
+    for i in range(hvdc_status.shape[0]):
+        from_idx = hvdc_bus_f[i]
+        to_idx = hvdc_bus_t[i]
+        for t in range(ntime):
+            if hvdc_status[i, t] != 0:
+                if used[from_idx, t] == 0:
+                    V[from_idx, t] = complex(hvdc_vf[i, t], 0)
+                    used[from_idx, t] = 1
+                if used[to_idx, t] == 0:
+                    V[to_idx, t] = complex(hvdc_vt[i, t], 0)
+                    used[to_idx, t] = 1
+
+    # branch - from
+    for i in iBeqv:  # branches controlling Vf
+        from_idx = VfBeqbus[i]
+        for t in range(ntime):
+            if branch_status[i, t] != 0:
+                if used[from_idx, t] == 0:
+                    V[from_idx, t] = complex(br_vf[i, t], 0)
+                    used[from_idx, t] = 1
+
+    # branch - to
+    for i in iVtma:  # branches controlling Vt
+        from_idx = Vtmabus[i]
+        for t in range(ntime):
+            if branch_status[i, t] != 0:
+                if used[from_idx, t] == 0:
+                    V[from_idx, t] = complex(br_vt[i, t], 0)
+                    used[from_idx, t] = 1
+
+    return V
+
+
 class SnapshotData:
 
     def __init__(self, nbus, nline, ndcline, ntr, nvsc, nupfc, nhvdc, nload, ngen, nbatt, nshunt, nstagen, sbase, ntime=1):
@@ -151,6 +240,8 @@ class SnapshotData:
         self.pv_ = None
         self.vd_ = None
         self.pqpv_ = None
+        self.ac_ = None
+        self.dc_ = None
 
         self.available_structures = ['Vbus',
                                      'Sbus',
@@ -158,6 +249,8 @@ class SnapshotData:
                                      'Ybus',
                                      'Yf',
                                      'Yt',
+                                     'Bbus',
+                                     'Bf',
                                      'Cf',
                                      'Ct',
                                      'Yshunt',
@@ -271,6 +364,29 @@ class SnapshotData:
         self.bus_data.bus_installed_power = self.generator_data.get_installed_power_per_bus()
         self.bus_data.bus_installed_power += self.battery_data.get_installed_power_per_bus()
 
+        self.bus_data.Vbus = compose_generator_voltage_profile(nbus=self.nbus,
+                                                               ntime=self.ntime,
+                                                               gen_bus_indices=self.generator_data.get_bus_indices(),
+                                                               gen_vset=self.generator_data.generator_v,
+                                                               gen_status=self.generator_data.generator_active,
+                                                               gen_is_controlled=self.generator_data.generator_controllable,
+                                                               bat_bus_indices=self.battery_data.get_bus_indices(),
+                                                               bat_vset=self.battery_data.battery_v,
+                                                               bat_status=self.battery_data.battery_active,
+                                                               bat_is_controlled=self.battery_data.battery_controllable,
+                                                               hvdc_bus_f=self.hvdc_data.get_bus_indices_f(),
+                                                               hvdc_bus_t=self.hvdc_data.get_bus_indices_t(),
+                                                               hvdc_status=self.hvdc_data.active,
+                                                               hvdc_vf=self.hvdc_data.Vset_f,
+                                                               hvdc_vt=self.hvdc_data.Vset_t,
+                                                               iBeqv=np.array(self.iBeqv, dtype=int),
+                                                               iVtma=np.array(self.iVtma, dtype=int),
+                                                               VfBeqbus=np.array(self.VfBeqbus, dtype=int),
+                                                               Vtmabus=np.array(self.Vtmabus, dtype=int),
+                                                               branch_status=self.branch_data.branch_active,
+                                                               br_vf=self.branch_data.vf_set,
+                                                               br_vt=self.branch_data.vt_set)
+
         self.determine_control_indices()
 
     def re_calc_admittance_matrices(self, tap_module, t=0, idx=None):
@@ -347,6 +463,7 @@ class SnapshotData:
         self.iQtma = list()  # indices of the branches controlling the Qt flow with ma
         self.iPfdp = list()  # indices of the drop converters controlling the power flow with theta sh
         self.iVscL = list()  # indices of the converters
+        self.iPfdp_va = list()
 
         self.any_control = False
 
@@ -464,8 +581,6 @@ class SnapshotData:
 
         # FUBM- Saves the "from" bus identifier for Vf controlled by Beq
         #  (Converters type II for Vdc control)
-        # self.VfBeqbus = np.unique(np.r_[VfBeqbus_sh, self.F[self.iBeqv]])
-        # self.VfBeqbus.sort()
         self.VfBeqbus = self.F[self.iBeqv]
 
         # FUBM- Saves the "to"   bus identifier for Vt controlled by ma
@@ -502,12 +617,16 @@ class SnapshotData:
     def Vbus(self):
 
         if self.Vbus_ is None:
-            self.Vbus_ = self.bus_data.Vbus.copy()
+            self.Vbus_ = self.bus_data.Vbus
 
         return self.Vbus_[:, 0]
 
     @property
     def Sbus(self):
+        """
+        Returns the power injections in per-unit
+        :return: array of power injections (p.u.)
+        """
 
         if self.Sbus_ is None:
             self.Sbus_ = self.get_injections(normalize=True)
@@ -526,6 +645,10 @@ class SnapshotData:
     def Rates(self):
         return self.branch_data.branch_rates[:, 0]
 
+    @property
+    def ContingencyRates(self):
+        return self.branch_data.branch_contingency_rates[:, 0]
+
     @property
     def Qmax_bus(self):
 
@@ -663,6 +786,21 @@ class SnapshotData:
     def hvdc_losses(self):
         return self.hvdc_data.get_losses()[:, 0]
 
+    @property
+    def ac_indices(self):
+
+        if self.ac_ is None:
+            self.ac_ = self.branch_data.get_ac_indices()
+
+        return self.ac_
+
+    @property
+    def dc_indices(self):
+        if self.dc_ is None:
+            self.dc_ = self.branch_data.get_dc_indices()
+
+        return self.dc_
+
     @property
     def Cf(self):
 
@@ -671,6 +809,10 @@ class SnapshotData:
             self.Cf_, self.Ct_ = ycalc.compute_connectivity(branch_active=self.branch_data.branch_active[:, 0],
                                                             Cf_=self.branch_data.C_branch_bus_f,
                                                             Ct_=self.branch_data.C_branch_bus_t)
+
+        if not isinstance(self.Cf_, sp.csc_matrix):
+            self.Cf_ = self.Cf_.tocsc()
+
         return self.Cf_
 
     @property
@@ -681,6 +823,10 @@ class SnapshotData:
             self.Cf_, self.Ct_ = ycalc.compute_connectivity(branch_active=self.branch_data.branch_active[:, 0],
                                                             Cf_=self.branch_data.C_branch_bus_f,
                                                             Ct_=self.branch_data.C_branch_bus_t)
+
+        if not isinstance(self.Ct_, sp.csc_matrix):
+            self.Ct_ = self.Ct_.tocsc()
+
         return self.Ct_
 
     @property
@@ -759,6 +905,10 @@ class SnapshotData:
 
         return self.Yshunt_
 
+    # @property
+    # def YshuntHelm(self):
+    #     return self.Yshunt_from_devices[:, 0]
+
     @property
     def B1(self):
 
@@ -785,10 +935,15 @@ class SnapshotData:
     def Bbus(self):
 
         if self.Bbus_ is None:
-            self.Bbus_, self.Bf_ = ycalc.compute_linear_admittances(X=self.branch_data.X,
+            self.Bbus_, self.Bf_ = ycalc.compute_linear_admittances(nbr=self.nbr,
+                                                                    X=self.branch_data.X,
+                                                                    R=self.branch_data.R,
                                                                     m=self.branch_data.m[:, 0],
+                                                                    active=self.branch_data.branch_active[:, 0],
                                                                     Cf=self.Cf,
-                                                                    Ct=self.Ct)
+                                                                    Ct=self.Ct,
+                                                                    ac=self.ac_indices,
+                                                                    dc=self.dc_indices)
             self.Bpqpv_ = self.Bbus_[np.ix_(self.pqpv, self.pqpv)]
             self.Bref_ = self.Bbus_[np.ix_(self.pqpv, self.vd)]
 
@@ -931,6 +1086,16 @@ class SnapshotData:
                               columns=self.bus_data.bus_names,
                               index=self.branch_data.branch_names)
 
+        elif structure_type == 'Bbus':
+            df = pd.DataFrame(data=self.Bbus.toarray(),
+                              columns=self.bus_data.bus_names,
+                              index=self.bus_data.bus_names)
+
+        elif structure_type == 'Bf':
+            df = pd.DataFrame(data=self.Bf.toarray(),
+                              columns=self.bus_data.bus_names,
+                              index=self.branch_data.branch_names)
+
         elif structure_type == 'Cf':
             df = pd.DataFrame(data=self.Cf.toarray(),
                               columns=self.bus_data.bus_names,
@@ -1006,7 +1171,8 @@ class SnapshotData:
 
         elif structure_type == 'Jacobian':
 
-            J = Jacobian(self.Ybus, self.Vbus, self.Ibus, self.pq, self.pqpv)
+            pvpq = np.r_[self.pv, self.pq]
+            J = Jacobian(self.Ybus, self.Vbus, self.Ibus, self.pq, pvpq)
 
             """
             J11 = dS_dVa[array([pvpq]).T, pvpq].real
@@ -1168,7 +1334,7 @@ class SnapshotData:
 
 def compile_snapshot_circuit(circuit: MultiCircuit, apply_temperature=False,
                              branch_tolerance_mode=BranchImpedanceMode.Specified,
-                             opf_results = None) -> SnapshotData:
+                             opf_results=None) -> SnapshotData:
     """
 
     :param circuit:
