@@ -31,7 +31,8 @@ class DcLine(EditableDevice):
                  rate=1.0, active=True, tolerance=0, cost=0.0,
                  mttf=0, mttr=0, r_fault=0.0, x_fault=0.0, fault_pos=0.5,
                  length=1, temp_base=20, temp_oper=20, alpha=0.00330,
-                 template=None, rate_prof=None, Cost_prof=None, active_prof=None, temp_oper_prof=None):
+                 template=None, rate_prof=None, Cost_prof=None, active_prof=None, temp_oper_prof=None,
+                 contingency_factor=1.0, contingency_enabled=True, monitor_loading=True):
         """
 
         :param bus_from:
@@ -72,7 +73,15 @@ class DcLine(EditableDevice):
                                                   'bus_to': GCProp('', DeviceType.BusDevice,
                                                                    'Name of the bus at the "to" side of the line.'),
                                                   'active': GCProp('', bool, 'Is the line active?'),
+
                                                   'rate': GCProp('MW', float, 'Thermal rating power of the line.'),
+
+                                                  'contingency_factor': GCProp('p.u.', float,
+                                                                               'Rating multiplier for contingencies.'),
+                                                  'contingency_enabled': GCProp('', bool,
+                                                                                'Consider this line for contingencies.'),
+                                                  'monitor_loading': GCProp('', bool,
+                                                                            'Monitor this device loading for optimization, NTC or contingency studies.'),
                                                   'mttf': GCProp('h', float, 'Mean time to failure, '
                                                                  'used in reliability studies.'),
                                                   'mttr': GCProp('h', float, 'Mean time to recovery, '
@@ -156,7 +165,9 @@ class DcLine(EditableDevice):
 
         # line rating in MW
         self.rate = rate
-
+        self.contingency_factor = contingency_factor
+        self.contingency_enabled: bool = contingency_enabled
+        self.monitor_loading: bool = monitor_loading
         self.rate_prof = rate_prof
 
         # branch type: Line, Transformer, etc...
@@ -307,32 +318,50 @@ class DcLine(EditableDevice):
             data.append(obj)
         return data
 
-    def get_properties_dict(self):
+    def get_properties_dict(self, version=3):
         """
         Get json dictionary
         :return:
         """
+        if version == 2:
+            return {'id': self.idtag,
+                    'type': 'dc_line',
+                    'phases': 'ps',
+                    'name': self.name,
+                    'name_code': self.code,
+                    'bus_from': self.bus_from.idtag,
+                    'bus_to': self.bus_to.idtag,
+                    'active': self.active,
+                    'rate': self.rate,
+                    'r': self.R,
+                    'length': self.length,
+                    'base_temperature': self.temp_base,
+                    'operational_temperature': self.temp_oper,
+                    'alpha': self.alpha,
+                    'locations': []}
+        elif version == 3:
+            return {'id': self.idtag,
+                    'type': 'dc_line',
+                    'phases': 'ps',
+                    'name': self.name,
+                    'name_code': self.code,
+                    'bus_from': self.bus_from.idtag,
+                    'bus_to': self.bus_to.idtag,
+                    'active': self.active,
+                    'rate': self.rate,
+                    'contingency_factor1': self.contingency_factor,
+                    'contingency_factor2': self.contingency_factor,
+                    'contingency_factor3': self.contingency_factor,
+                    'r': self.R,
+                    'length': self.length,
+                    'base_temperature': self.temp_base,
+                    'operational_temperature': self.temp_oper,
+                    'alpha': self.alpha,
+                    'locations': []}
+        else:
+            return dict()
 
-        d = {'id': self.idtag,
-             'type': 'dc_line',
-             'phases': 'ps',
-             'name': self.name,
-             'name_code': self.code,
-             'bus_from': self.bus_from.idtag,
-             'bus_to': self.bus_to.idtag,
-             'active': self.active,
-
-             'rate': self.rate,
-             'r': self.R,
-             'length': self.length,
-             'base_temperature': self.temp_base,
-             'operational_temperature': self.temp_oper,
-             'alpha': self.alpha,
-             'locations': []}
-
-        return d
-
-    def get_profiles_dict(self):
+    def get_profiles_dict(self, version=3):
 
         if self.active_prof is not None:
             active_prof = self.active_prof.tolist()
@@ -345,7 +374,7 @@ class DcLine(EditableDevice):
                 'active': active_prof,
                 'rate': rate_prof}
 
-    def get_units_dict(self):
+    def get_units_dict(self, version=3):
         """
         Get units of the values
         """
