@@ -24,7 +24,7 @@ from GridCal.Engine.Devices.enumerations import BranchType
 from GridCal.Engine.Devices.underground_line import UndergroundLineType
 
 from GridCal.Engine.Devices.editable_device import EditableDevice, DeviceType, GCProp
-from GridCal.Engine.Devices.tower import Tower
+from GridCal.Engine.Devices.enumerations import HvdcControlType
 
 
 def firing_angles_to_reactive_limits(P, alphamin, alphamax):
@@ -156,10 +156,11 @@ class HvdcLine(EditableDevice):
     """
 
     def __init__(self, bus_from: Bus = None, bus_to: Bus = None, name='HVDC Line', idtag=None, active=True,
-                 rate=1.0, Pset=0.0, loss_factor=0.0, Vset_f=1.0, Vset_t=1.0, length=1.0, mttf=0.0, mttr=0.0,
+                 rate=1.0, Pset=0.0, r=1e-20, loss_factor=0.0, Vset_f=1.0, Vset_t=1.0, length=1.0, mttf=0.0, mttr=0.0,
                  overload_cost=1000.0,   min_firing_angle_f=-1.0, max_firing_angle_f=1.0, min_firing_angle_t=-1.0,
                  max_firing_angle_t=1.0, active_prof=np.ones(0, dtype=bool), rate_prof=np.zeros(0),
-                 Pset_prof=np.zeros(0), Vset_f_prof=np.ones(0), Vset_t_prof=np.ones(0), overload_cost_prof=np.zeros(0)):
+                 Pset_prof=np.zeros(0), Vset_f_prof=np.ones(0), Vset_t_prof=np.ones(0), overload_cost_prof=np.zeros(0),
+                 contingency_factor=1.0, control_mode: HvdcControlType=HvdcControlType.type_1_Pset):
         """
         HVDC Line model
         :param bus_from: Bus from
@@ -203,8 +204,15 @@ class HvdcLine(EditableDevice):
 
                                                   'rate': GCProp('MVA', float, 'Thermal rating power of the line.'),
 
+                                                  'contingency_factor': GCProp('p.u.', float,
+                                                                               'Rating multiplier for contingencies.'),
+
+                                                  'control_mode': GCProp('-', HvdcControlType, 'Control type.'),
+
                                                   'Pset': GCProp('MW', float, 'Set power flow.'),
 
+                                                  'r': GCProp('Ohm', float, 'line resistance.'),
+
                                                   'loss_factor': GCProp('p.u.', float,
                                                                         'Losses factor.\n'
                                                                         'The losses are computed as losses=Pfset x Ploss'),
@@ -256,6 +264,8 @@ class HvdcLine(EditableDevice):
 
         self.Pset = Pset
 
+        self.r = r
+
         self.loss_factor = loss_factor
 
         self.mttf = mttf
@@ -283,6 +293,8 @@ class HvdcLine(EditableDevice):
 
         self.overload_cost_prof = overload_cost_prof
 
+        self.control_mode = control_mode
+
         self.Pset_prof = Pset_prof
         self.active_prof = active_prof
         self.Vset_f_prof = Vset_f_prof
@@ -290,6 +302,7 @@ class HvdcLine(EditableDevice):
 
         # branch rating in MVA
         self.rate = rate
+        self.contingency_factor = contingency_factor
         self.rate_prof = rate_prof
 
     def get_from_and_to_power(self):
@@ -404,41 +417,70 @@ class HvdcLine(EditableDevice):
             data.append(obj)
         return data
 
-    def get_properties_dict(self):
+    def get_properties_dict(self, version=3):
         """
         Get json dictionary
         :return:
         """
+        if version == 2:
+            return {'id': self.idtag,
+                    'type': 'hvdc',
+                    'phases': 'ps',
+                    'name': self.name,
+                    'name_code': self.code,
+                    'bus_from': self.bus_from.idtag,
+                    'bus_to': self.bus_to.idtag,
+                    'active': self.active,
+                    'rate': self.rate,
+                    'r': 0,
+                    'length': self.length,
+                    'loss_factor': self.loss_factor,
+                    'vset_from': self.Vset_f,
+                    'vset_to': self.Vset_t,
+                    'Pset': self.Pset,
+                    'min_firing_angle_f': self.min_firing_angle_f,
+                    'max_firing_angle_f': self.max_firing_angle_f,
+                    'min_firing_angle_t': self.min_firing_angle_t,
+                    'max_firing_angle_t': self.max_firing_angle_t,
+                    'overload_cost': self.overload_cost,
+                    'base_temperature': 20,
+                    'operational_temperature': 20,
+                    'alpha': 0.00330,
+                    'locations': []
+                    }
+        elif version == 3:
+            return {'id': self.idtag,
+                    'type': 'hvdc',
+                    'phases': 'ps',
+                    'name': self.name,
+                    'name_code': self.code,
+                    'bus_from': self.bus_from.idtag,
+                    'bus_to': self.bus_to.idtag,
+                    'active': self.active,
+                    'rate': self.rate,
+                    'contingency_factor1': self.contingency_factor,
+                    'contingency_factor2': self.contingency_factor,
+                    'contingency_factor3': self.contingency_factor,
+                    'r': 0,
+                    'length': self.length,
+                    'loss_factor': self.loss_factor,
+                    'vset_from': self.Vset_f,
+                    'vset_to': self.Vset_t,
+                    'Pset': self.Pset,
+                    'min_firing_angle_f': self.min_firing_angle_f,
+                    'max_firing_angle_f': self.max_firing_angle_f,
+                    'min_firing_angle_t': self.min_firing_angle_t,
+                    'max_firing_angle_t': self.max_firing_angle_t,
+                    'overload_cost': self.overload_cost,
+                    'base_temperature': 20,
+                    'operational_temperature': 20,
+                    'alpha': 0.00330,
+                    'locations': []
+                    }
+        else:
+            return dict()
 
-        d = {'id': self.idtag,
-             'type': 'hvdc',
-             'phases': 'ps',
-             'name': self.name,
-             'name_code': self.code,
-             'bus_from': self.bus_from.idtag,
-             'bus_to': self.bus_to.idtag,
-             'active': self.active,
-             'rate': self.rate,
-             'r': 0,
-             'length': self.length,
-             'loss_factor': self.loss_factor,
-             'vset_from': self.Vset_f,
-             'vset_to': self.Vset_t,
-             'Pset': self.Pset,
-             'min_firing_angle_f': self.min_firing_angle_f,
-             'max_firing_angle_f': self.max_firing_angle_f,
-             'min_firing_angle_t': self.min_firing_angle_t,
-             'max_firing_angle_t': self.max_firing_angle_t,
-             'overload_cost': self.overload_cost,
-             'base_temperature': 20,
-             'operational_temperature': 20,
-             'alpha': 0.00330,
-             'locations': []
-             }
-
-        return d
-
-    def get_profiles_dict(self):
+    def get_profiles_dict(self, version=3):
         """
 
         :return:
@@ -467,7 +509,7 @@ class HvdcLine(EditableDevice):
                 'vset_to': vset_prof_t,
                 'overload_cost': cost_prof}
 
-    def get_units_dict(self):
+    def get_units_dict(self, version=3):
         """
         Get units of the values
         """
