@@ -60,7 +60,7 @@ class Load(EditableDevice):
 
     """
 
-    def __init__(self, name='Load', idtag=None, code='', G=0.0, B=0.0, Ir=0.0, Ii=0.0, P=0.0, Q=0.0, cost=0.0,
+    def __init__(self, name='Load', idtag=None, code='', G=0.0, B=0.0, Ir=0.0, Ii=0.0, P=0.0, Q=0.0, cost=1200.0,
                  G_prof=None, B_prof=None, Ir_prof=None, Ii_prof=None, P_prof=None, Q_prof=None,
                  active=True, mttf=0.0, mttr=0.0):
 
@@ -163,31 +163,30 @@ class Load(EditableDevice):
 
         return load
 
-    def get_properties_dict(self):
+    def get_properties_dict(self, version=3):
         """
         Get json dictionary
         :return:
         """
+        if version in [2, 3]:
+            return {'id': self.idtag,
+                    'type': 'load',
+                    'phases': 'ps',
+                    'name': self.name,
+                    'name_code': self.code,
+                    'bus': self.bus.idtag,
+                    'active': self.active,
+                    'g': self.G,
+                    'b': self.B,
+                    'ir': self.Ir,
+                    'ii': self.Ii,
+                    'p': self.P,
+                    'q': self.Q
+                    }
+        else:
+            return dict()
 
-        d = {'id': self.idtag,
-             'type': 'load',
-             'phases': 'ps',
-             'name': self.name,
-             'name_code': self.code,
-             'bus': self.bus.idtag,
-             'active': self.active,
-
-             'g': self.G,
-             'b': self.B,
-             'ir': self.Ir,
-             'ii': self.Ii,
-             'p': self.P,
-             'q': self.Q
-             }
-
-        return d
-
-    def get_profiles_dict(self):
+    def get_profiles_dict(self, version=3):
         """
 
         :return:
@@ -219,7 +218,7 @@ class Load(EditableDevice):
                 'g': G_prof,
                 'b': B_prof}
 
-    def get_units_dict(self):
+    def get_units_dict(self, version=3):
         """
         Get units of the values
         """
