@@ -172,7 +172,11 @@ class TransformerType(EditableDevice):
 
         Zseries = ZseriesHv / ZbaseHvSys + ZseriesLv / ZbaseLvSys
         Zshunt = ZshuntHv / ZbaseHvSys + ZshuntLv / ZbaseLvSys
-        Yshunt = 1 / Zshunt
+
+        if Zshunt != 0:
+            Yshunt = 1 / Zshunt
+        else:
+            Yshunt = 0j
 
         return Zseries, Yshunt
 
@@ -414,13 +418,14 @@ class Transformer2W(EditableDevice):
                  rate=1.0,
                  tap=1.0, tap_module_max=1.2, tap_module_min=0.5,
                  shift_angle=0.0, theta_max=6.28, theta_min=-6.28,
-                 active=True, tolerance=0, cost=0.0,
+                 active=True, tolerance=0, cost=100.0,
                  mttf=0, mttr=0,
                  vset=1.0, Pset=0, bus_to_regulated=False,
                  temp_base=20, temp_oper=20, alpha=0.00330,
                  control_mode: TransformerControlType = TransformerControlType.fixed,
                  template: TransformerType = None,
-                 rate_prof=None, Cost_prof=None, active_prof=None, temp_oper_prof=None):
+                 rate_prof=None, Cost_prof=None, active_prof=None, temp_oper_prof=None, contingency_factor=1.0,
+                 contingency_enabled=True, monitor_loading=True):
 
         EditableDevice.__init__(self,
                                 name=name,
@@ -438,7 +443,15 @@ class Transformer2W(EditableDevice):
                                                   'active': GCProp('', bool, 'Is the branch active?'),
                                                   'HV': GCProp('kV', float, 'High voltage rating'),
                                                   'LV': GCProp('kV', float, 'Low voltage rating'),
+
                                                   'rate': GCProp('MVA', float, 'Thermal rating power of the branch.'),
+
+                                                  'contingency_factor': GCProp('p.u.', float,
+                                                                               'Rating multiplier for contingencies.'),
+                                                  'contingency_enabled': GCProp('', bool,
+                                                                                'Consider this transformer for contingencies.'),
+                                                  'monitor_loading': GCProp('', bool,
+                                                                            'Monitor this device loading for optimization, NTC or contingency studies.'),
                                                   'mttf': GCProp('h', float, 'Mean time to failure, '
                                                                  'used in reliability studies.'),
                                                   'mttr': GCProp('h', float, 'Mean time to recovery, '
@@ -560,7 +573,9 @@ class Transformer2W(EditableDevice):
 
         # branch rating in MVA
         self.rate = rate
-
+        self.contingency_factor = contingency_factor
+        self.contingency_enabled: bool = contingency_enabled
+        self.monitor_loading: bool = monitor_loading
         self.rate_prof = rate_prof
 
         # branch type: Line, Transformer, etc...
@@ -653,6 +668,11 @@ class Transformer2W(EditableDevice):
 
         return b
 
+    def flip(self):
+
+        F, T = self.bus_from, self.bus_to
+        self.bus_to, self.bus_from = F, T
+
     def tap_up(self):
         """
         Move the tap changer one position up
@@ -803,7 +823,7 @@ class Transformer2W(EditableDevice):
             data.append(obj)
         return data
 
-    def get_properties_dict(self):
+    def get_properties_dict(self, version=3):
         """
         Get json dictionary
         :return:
@@ -830,53 +850,103 @@ class Transformer2W(EditableDevice):
                          TransformerControlType.PtVt: 3,
                          TransformerControlType.Qt: 4,
                          TransformerControlType.PtQt: 5}
+        if version == 2:
+            d = {'id': self.idtag,
+                 'type': 'transformer',
+                 'phases': 'ps',
+                 'name': self.name,
+                 'name_code': self.code,
+                 'bus_from': self.bus_from.idtag,
+                 'bus_to': self.bus_to.idtag,
+                 'active': self.active,
+                 'rate': self.rate,
+                 'Vnomf': v_from,
+                 'Vnomt': v_to,
+                 'r': self.R,
+                 'x': self.X,
+                 'g': self.G,
+                 'b': self.B,
+                 'tap_module': self.tap_module,
+                 'min_tap_module': self.tap_module_min,
+                 'max_tap_module': self.tap_module_max,
+                 'id_tap_module_table': "",
 
-        d = {'id': self.idtag,
-             'type': 'transformer',
-             'phases': 'ps',
-             'name': self.name,
-             'name_code': self.code,
-             'bus_from': self.bus_from.idtag,
-             'bus_to': self.bus_to.idtag,
-             'active': self.active,
-             'rate': self.rate,
-             'Vnomf': v_from,
-             'Vnomt': v_to,
-             'r': self.R,
-             'x': self.X,
-             'g': self.G,
-             'b': self.B,
-             'tap_module': self.tap_module,
-             'min_tap_module': self.tap_module_min,
-             'max_tap_module': self.tap_module_max,
-             'id_tap_module_table': "",
+                 'tap_angle': self.angle,
+                 'min_tap_angle': self.angle_min,
+                 'max_tap_angle': self.angle_max,
+                 'id_tap_angle_table': "",
 
-             'tap_angle': self.angle,
-             'min_tap_angle': self.angle_min,
-             'max_tap_angle': self.angle_max,
-             'id_tap_angle_table': "",
+                 'control_mode': control_modes[self.control_mode],
 
-             'control_mode': control_modes[self.control_mode],
+                 # 'min_tap_position': self.tap_changer.min_tap,
+                 # 'max_tap_position': self.tap_changer.max_tap,
+                 # 'tap_inc_reg_down': self.tap_changer.inc_reg_down,
+                 # 'tap_inc_reg_up': self.tap_changer.inc_reg_up,
+                 # 'virtual_tap_from': tap_f,
+                 # 'virtual_tap_to': tap_t,
+                 # 'bus_to_regulated': self.bus_to_regulated,
 
-             # 'min_tap_position': self.tap_changer.min_tap,
-             # 'max_tap_position': self.tap_changer.max_tap,
-             # 'tap_inc_reg_down': self.tap_changer.inc_reg_down,
-             # 'tap_inc_reg_up': self.tap_changer.inc_reg_up,
-             # 'virtual_tap_from': tap_f,
-             # 'virtual_tap_to': tap_t,
-             # 'bus_to_regulated': self.bus_to_regulated,
+                 'vset': self.vset,
+                 'pset': self.Pset,
 
-             'vset': self.vset,
-             'pset': self.Pset,
+                 'base_temperature': self.temp_base,
+                 'operational_temperature': self.temp_oper,
+                 'alpha': self.alpha
+                 }
 
-             'base_temperature': self.temp_base,
-             'operational_temperature': self.temp_oper,
-             'alpha': self.alpha
-             }
+        elif version == 3:
+            d = {'id': self.idtag,
+                 'type': 'transformer',
+                 'phases': 'ps',
+                 'name': self.name,
+                 'name_code': self.code,
+                 'bus_from': self.bus_from.idtag,
+                 'bus_to': self.bus_to.idtag,
+                 'active': self.active,
+                 'rate': self.rate,
+                 'contingency_factor1': self.contingency_factor,
+                 'contingency_factor2': self.contingency_factor,
+                 'contingency_factor3': self.contingency_factor,
+
+                 'Vnomf': v_from,
+                 'Vnomt': v_to,
+                 'r': self.R,
+                 'x': self.X,
+                 'g': self.G,
+                 'b': self.B,
+                 'tap_module': self.tap_module,
+                 'min_tap_module': self.tap_module_min,
+                 'max_tap_module': self.tap_module_max,
+                 'id_tap_module_table': "",
+
+                 'tap_angle': self.angle,
+                 'min_tap_angle': self.angle_min,
+                 'max_tap_angle': self.angle_max,
+                 'id_tap_angle_table': "",
+
+                 'control_mode': control_modes[self.control_mode],
+
+                 # 'min_tap_position': self.tap_changer.min_tap,
+                 # 'max_tap_position': self.tap_changer.max_tap,
+                 # 'tap_inc_reg_down': self.tap_changer.inc_reg_down,
+                 # 'tap_inc_reg_up': self.tap_changer.inc_reg_up,
+                 # 'virtual_tap_from': tap_f,
+                 # 'virtual_tap_to': tap_t,
+                 # 'bus_to_regulated': self.bus_to_regulated,
+
+                 'vset': self.vset,
+                 'pset': self.Pset,
+
+                 'base_temperature': self.temp_base,
+                 'operational_temperature': self.temp_oper,
+                 'alpha': self.alpha
+                 }
+        else:
+            d = dict()
 
         return d
 
-    def get_profiles_dict(self):
+    def get_profiles_dict(self, version=3):
         """
 
         :return:
@@ -892,7 +962,7 @@ class Transformer2W(EditableDevice):
                 'active': active_prof,
                 'rate': rate_prof}
 
-    def get_units_dict(self):
+    def get_units_dict(self, version=3):
         """
         Get units of the values
         """
