@@ -29,13 +29,11 @@ from GridCal.Engine.IO.ipa_parser import load_iPA
 from GridCal.Engine.IO.json_parser import parse_json, parse_json_data_v2, parse_json_data_v3
 from GridCal.Engine.IO.raw_parser import PSSeParser
 from GridCal.Engine.IO.cim.cim_parser import CIMImport
-from GridCal.Engine.IO.zip_interface import save_data_frames_to_zip, get_frames_from_zip
+from GridCal.Engine.IO.zip_interface import save_data_frames_to_zip, get_frames_from_zip, get_session_tree, load_session_driver_objects
 from GridCal.Engine.IO.sqlite_interface import save_data_frames_to_sqlite, open_data_frames_from_sqlite
 from GridCal.Engine.IO.h5_interface import save_h5, open_h5
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
 
-from PySide2.QtCore import QThread, Signal
-
 
 class FileOpen:
 
@@ -105,7 +103,8 @@ class FileOpen:
                     # open file content
                     data_dictionary = get_frames_from_zip(self.file_name,
                                                           text_func=text_func,
-                                                          progress_func=progress_func)
+                                                          progress_func=progress_func,
+                                                          logger=self.logger)
                     # interpret file content
                     if data_dictionary is not None:
                         self.circuit = data_frames_to_circuit(data_dictionary)
@@ -187,7 +186,8 @@ class FileOpen:
 
 class FileSave:
 
-    def __init__(self, circuit: MultiCircuit, file_name, text_func=None, progress_func=None, simulation_drivers=list()):
+    def __init__(self, circuit: MultiCircuit, file_name, text_func=None, progress_func=None,
+                 simulation_drivers=list(), sessions=list()):
         """
         File saver
         :param circuit: MultiCircuit
@@ -202,6 +202,8 @@ class FileSave:
 
         self.simulation_drivers = simulation_drivers
 
+        self.sessions = sessions
+
         self.text_func = text_func
 
         self.progress_func = progress_func
@@ -255,8 +257,9 @@ class FileSave:
 
         dfs = create_data_frames(self.circuit)
 
-        save_data_frames_to_zip(dfs,
+        save_data_frames_to_zip(dfs=dfs,
                                 filename_zip=self.file_name,
+                                sessions=self.sessions,
                                 text_func=self.text_func,
                                 progress_func=self.progress_func)
 
@@ -311,115 +314,3 @@ class FileSave:
 
         return logger
 
-
-class FileOpenThread(QThread):
-    progress_signal = Signal(float)
-    progress_text = Signal(str)
-    done_signal = Signal()
-
-    def __init__(self, file_name):
-        """
-        Constructor
-        :param file_name: file name were to save
-        """
-        QThread.__init__(self)
-
-        self.file_name = file_name
-
-        self.valid = False
-
-        self.logger = Logger()
-
-        self.circuit = None
-
-        self.__cancel__ = False
-
-    def run(self):
-        """
-        run the file open procedure
-        """
-        self.circuit = MultiCircuit()
-
-        if isinstance(self.file_name, list):
-            path, fname = os.path.split(self.file_name[0])
-            self.progress_text.emit('Loading ' + fname + '...')
-        else:
-            path, fname = os.path.split(self.file_name)
-            self.progress_text.emit('Loading ' + fname + '...')
-
-        self.logger = Logger()
-
-        file_handler = FileOpen(file_name=self.file_name)
-
-        self.circuit = file_handler.open(text_func=self.progress_text.emit,
-                                         progress_func=self.progress_signal.emit)
-
-        self.logger += file_handler.logger
-        self.valid = True
-
-        # post events
-        self.progress_text.emit('Done!')
-
-        self.done_signal.emit()
-
-    def cancel(self):
-        self.__cancel__ = True
-
-
-class FileSaveThread(QThread):
-    progress_signal = Signal(float)
-    progress_text = Signal(str)
-    done_signal = Signal()
-
-    def __init__(self, circuit: MultiCircuit, file_name, simulation_drivers=list()):
-        """
-        Constructor
-        :param circuit: MultiCircuit instance
-        :param file_name: name of the file where to save
-        :param simulation_drivers: List of Simulation Drivers
-        """
-        QThread.__init__(self)
-
-        self.circuit = circuit
-
-        self.file_name = file_name
-
-        self.valid = False
-
-        self.simulation_drivers = simulation_drivers
-
-        self.logger = Logger()
-
-        self.error_msg = ''
-
-        self.__cancel__ = False
-
-    def run(self):
-        """
-        run the file save procedure
-        @return:
-        """
-
-        path, fname = os.path.split(self.file_name)
-
-        self.progress_text.emit('Flushing ' + fname + ' into ' + fname + '...')
-
-        self.logger = Logger()
-
-        file_handler = FileSave(self.circuit,
-                                self.file_name,
-                                text_func=self.progress_text.emit,
-                                progress_func=self.progress_signal.emit,
-                                simulation_drivers=self.simulation_drivers)
-
-        self.logger = file_handler.save()
-
-        self.valid = True
-
-        # post events
-        self.progress_text.emit('Done!')
-
-        self.done_signal.emit()
-
-    def cancel(self):
-        self.__cancel__ = True
\ No newline at end of file
