@@ -83,6 +83,26 @@ class PSSeGrid:
         area_dict = {val.I: elm for val, elm in zip(self.areas, circuit.areas)}
         zones_dict = {val.I: elm for val, elm in zip(self.zones, circuit.zones)}
 
+        # scan for missing zones or areas (yes, PSSe is so crappy that can reference areas that do not exist)
+        missing_areas = False
+        missing_zones = False
+        for psse_bus in self.buses:
+
+            # replace area idx by area name if available
+            if abs(psse_bus.bus.area) not in area_dict.keys():
+                area_dict[abs(psse_bus.bus.area)] = Area(name='A' + str(abs(psse_bus.bus.area)))
+                missing_areas = True
+
+            if abs(psse_bus.bus.zone) not in zones_dict.keys():
+                zones_dict[abs(psse_bus.bus.zone)] = Zone(name='Z' + str(abs(psse_bus.bus.zone)))
+                missing_zones = True
+
+        if missing_areas:
+            circuit.areas = [v for k, v in area_dict.items()]
+
+        if missing_zones:
+            circuit.zones = [v for k, v in zones_dict.items()]
+
         # ---------------------------------------------------------------------
         # Bus related
         # ---------------------------------------------------------------------
@@ -258,13 +278,14 @@ class PSSeBus:
         if self.bus.type == BusMode.Slack:
             self.bus.is_slack = True
 
-        if int(self.IDE) == 4:
-            self.bus.active = False
-
         # Ensures unique name
         self.bus.name = self.bus.name.replace("'", "").strip()
+
         self.bus.code = str(self.I)
 
+        if self.bus.name == '':
+            self.bus.name = 'Bus ' + str(self.I)
+
 
 class PSSeLoad:
 
@@ -621,16 +642,25 @@ class PSSeBranch:
         j = abs(self.J)
         bus_from = psse_bus_dict[i]
         bus_to = psse_bus_dict[j]
-        name = str(i) + '_' + str(j) + '_' + str(self.CKT).replace("'", "").strip()
+        code = str(i) + '_' + str(j) + '_' + str(self.CKT).replace("'", "").strip()
+        name = "{0}_{1}_{2}_{3}_{4}_{5}_{6}".format(i, bus_from.name, bus_from.Vnom, j, bus_to.name, bus_to.Vnom, self.CKT)
+        name = name.replace("'", "").replace(" ", "").strip()
 
-        branch = Line(bus_from=bus_from, bus_to=bus_to,
+        contingency_factor = self.RATEB / self.RATEA if self.RATEA > 0.0 else 1.0
+
+        if contingency_factor == 0:
+            contingency_factor = 1.0
+
+        branch = Line(bus_from=bus_from,
+                      bus_to=bus_to,
                       idtag=None,
-                      code=name,
+                      code=code,
                       name=name,
                       r=self.R,
                       x=self.X,
                       b=self.B,
-                      rate=max(self.RATEA, self.RATEB, self.RATEC),
+                      rate=self.RATEA,
+                      contingency_factor=round(contingency_factor, 6),
                       active=bool(self.ST),
                       mttf=0,
                       mttr=0,
@@ -803,7 +833,13 @@ class PSSeVscDCLine:
         Vset_t = self.ACSET2
         rate = max(self.SMAX1, self.SMAX2)
 
-        specified_power = 0
+        # Estimate power
+        # P = dV^2 / R
+        V1 = bus1.Vnom * Vset_f
+        V2 = bus2.Vnom * Vset_t
+        dV = (V1-V2) * 1000.0  # in V
+        P = dV * dV / self.RDC  # power in W
+        specified_power = P * 1e-6  # power in MW
 
         obj = HvdcLine(bus_from=bus1,
                        bus_to=bus2,
@@ -1086,14 +1122,21 @@ class PSSeTransformer:
 
         self.NAME = self.NAME.replace("'", "").strip()
 
-        if self.NAME == '':
-            self.NAME = str(self.I) + '_' + str(self.J) + '_' + str(self.CKT)
-            self.NAME = self.NAME.strip()
-
         if self.windings == 2:
             bus_from = psse_bus_dict[self.I]
             bus_to = psse_bus_dict[self.J]
 
+            # 11000_AGUAYO_400_12004_ABANTO_400_1_CKT
+            # if self.NAME != "":
+            # name = "{0}:{1}_{2}_{3}_{4}_{5}_{6}_{7}".format(self.NAME,
+            #                                                 self.I, bus_from.name, bus_from.Vnom,
+            #                                                 self.J, bus_to.name, bus_to.Vnom, self.CKT)
+            # else:
+            name = "{0}_{1}_{2}_{3}_{4}_{5}_{6}".format(self.I, bus_from.name, bus_from.Vnom,
+                                                        self.J, bus_to.name, bus_to.Vnom, self.CKT)
+
+            name = name.replace("'", "").replace(" ", "").strip()
+
             code = str(self.I) + '_' + str(self.J) + '_' + str(self.CKT)
             code = code.strip().replace("'", "")
 
@@ -1138,20 +1181,21 @@ class PSSeTransformer:
             else:
                 V2 = self.NOMV2
 
-            # print(idtag, 'CM', self.CM, 'CW', self.CW, 'CZ', self.CZ)
+            contingency_factor = self.RATB1 / self.RATA1 if self.RATA1 > 0.0 else 1.0
 
             elm = Transformer2W(bus_from=bus_from,
                                 bus_to=bus_to,
                                 idtag=None,
                                 code=code,
-                                name=self.NAME,
+                                name=name,
                                 HV=V1,
                                 LV=V2,
                                 r=r,
                                 x=x,
                                 g=g,
                                 b=b,
-                                rate=max(self.RATA1, self.RATB1, self.RATC1),
+                                rate=self.RATA1,
+                                contingency_factor=round(contingency_factor, 6),
                                 tap=tap_mod,
                                 shift_angle=np.deg2rad(self.ANG1),
                                 active=bool(self.STAT),
@@ -1569,7 +1613,7 @@ class PSSeParser:
                 name = re.search(str_a + '(.*)' + str_b, srch).group(1).strip()
                 data2 = sections[i - 1].split('\n')[1:]
 
-                if name.lower() == 'bus':
+                if name.lower() == 'bus' and len(data2) > 1:
                     data2.pop(0)
                     data2.pop(0)
 
