@@ -18,18 +18,15 @@ import numpy as np
 import json
 from matplotlib import pyplot as plt
 
-from PySide2.QtCore import QThread, Signal
-
 from GridCal.Engine.Simulations.results_template import ResultsTemplate
 from GridCal.Engine.Simulations.PowerFlow.power_flow_worker import PowerFlowOptions
 from GridCal.Engine.Simulations.result_types import ResultTypes
 from GridCal.Engine.Simulations.ContinuationPowerFlow.continuation_power_flow import continuation_nr, CpfStopAt, CpfParametrization, CpfNumericResults
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
 from GridCal.Engine.Core.snapshot_pf_data import compile_snapshot_circuit
-from GridCal.Engine.plot_config import LINEWIDTH
-from GridCal.Engine.Simulations.results_model import ResultsModel
+from GridCal.Engine.Simulations.results_table import ResultsTable
 from GridCal.Engine.Simulations.driver_types import SimulationTypes
-
+from GridCal.Engine.Simulations.driver_template import DriverTemplate
 
 ########################################################################################################################
 # Voltage collapse classes
@@ -188,7 +185,7 @@ class ContinuationPowerFlowResults(ResultsTemplate):
         self.loading[np.ix_(nval, branch_original_idx)] = results.loading
         self.losses[np.ix_(nval, branch_original_idx)] = results.losses
 
-    def mdl(self, result_type: ResultTypes = ResultTypes.BusVoltage) -> ResultsModel:
+    def mdl(self, result_type: ResultTypes) -> ResultsTable:
         """
         Plot the results
         :param result_type:
@@ -283,15 +280,12 @@ class ContinuationPowerFlowResults(ResultsTemplate):
             y = self.voltages
 
         # assemble model
-        mdl = ResultsModel(data=y, index=x, columns=labels, title=title,
+        mdl = ResultsTable(data=y, index=x, columns=labels, title=title,
                            ylabel=y_label, xlabel=x_label, units=y_label)
         return mdl
 
 
-class ContinuationPowerFlowDriver(QThread):
-    progress_signal = Signal(float)
-    progress_text = Signal(str)
-    done_signal = Signal()
+class ContinuationPowerFlowDriver(DriverTemplate):
     name = 'Continuation Power Flow'
     tpe = SimulationTypes.ContinuationPowerFlow_run
 
@@ -309,10 +303,7 @@ class ContinuationPowerFlowDriver(QThread):
         :param opf_results:
         """
 
-        QThread.__init__(self)
-
-        # MultiCircuit instance
-        self.circuit = circuit
+        DriverTemplate.__init__(self, grid=circuit)
 
         # voltage stability options
         self.options = options
@@ -327,8 +318,6 @@ class ContinuationPowerFlowDriver(QThread):
 
         self.results = None
 
-        self.__cancel__ = False
-
     def get_steps(self):
         """
         List of steps
@@ -353,7 +342,7 @@ class ContinuationPowerFlowDriver(QThread):
         """
         print('Running voltage collapse...')
 
-        nc = compile_snapshot_circuit(circuit=self.circuit,
+        nc = compile_snapshot_circuit(circuit=self.grid,
                                       apply_temperature=self.pf_options.apply_temperature_correction,
                                       branch_tolerance_mode=self.pf_options.branch_impedance_tolerance_mode,
                                       opf_results=self.opf_results)
@@ -426,10 +415,3 @@ class ContinuationPowerFlowDriver(QThread):
         print('done!')
         self.progress_text.emit('Done!')
         self.done_signal.emit()
-
-    def cancel(self):
-        self.__cancel__ = True
-        self.progress_signal.emit(0.0)
-        self.progress_text.emit('Cancelled!')
-        self.done_signal.emit()
-
