@@ -154,37 +154,30 @@ def add_dc_nodal_power_balance(numerical_circuit: OpfTimeCircuit, problem: LpPro
 
 def add_branch_loading_restriction(problem: LpProblem,
                                    theta_f, theta_t, Bseries,
-                                   Fmax, FSlack1, FSlack2):
+                                   ratings, ratings_slack_from, ratings_slack_to):
     """
     Add the branch loading restrictions
     :param problem: LpProblem instance
     :param theta_f: voltage angles at the "from" side of the branches (m, nt)
     :param theta_t: voltage angles at the "to" side of the branches (m, nt)
     :param Bseries: Array of branch susceptances (m)
-    :param Fmax: Array of branch ratings (m, nt)
-    :param FSlack1: Array of branch loading slack variables in the from-to sense
-    :param FSlack2: Array of branch loading slack variables in the to-from sense
+    :param ratings: Array of branch ratings (m, nt)
+    :param ratings_slack_from: Array of branch loading slack variables in the from-to sense
+    :param ratings_slack_to: Array of branch loading slack variables in the to-from sense
     :return: Nothing
     """
-
-    load_f = Bseries * (theta_f - theta_t)
-    load_t = Bseries * (theta_t - theta_f)
+    m = ratings_slack_to.shape[1]
 
     # from-to branch power restriction
-    lpAddRestrictions2(problem=problem,
-                       lhs=load_f,
-                       rhs=np.array([Fmax[:, i] + FSlack1[:, i] for i in range(FSlack1.shape[1])]).transpose(),  # Fmax + FSlack1
-                       name='from_to_branch_rate',
-                       op='<=')
+    load_f = Bseries * (theta_f - theta_t)
 
-    # to-from branch power restriction
-    lpAddRestrictions2(problem=problem,
-                       lhs=load_t,
-                       rhs=np.array([Fmax[:, i] + FSlack2[:, i] for i in range(FSlack2.shape[1])]).transpose(),  # Fmax + FSlack2
-                       name='to_from_branch_rate',
-                       op='<=')
+    lpAddRestrictions3(problem=problem,
+                       lhs=np.array([-ratings[:, i] - ratings_slack_to[:, i] for i in range(m)]).transpose(),
+                       var=load_f,
+                       rhs=np.array([ratings[:, i] + ratings_slack_from[:, i] for i in range(m)]).transpose(),
+                       name='2_side_branch_rate')
 
-    return load_f, load_t
+    return load_f
 
 
 def add_battery_discharge_restriction(problem: LpProblem, SoC0, Capacity, Efficiency, Pb, E, dt):
@@ -234,18 +227,18 @@ class OpfDcTimeSeries(OpfTimeSeries):
         :param numerical_circuit: NumericalCircuit instance
         :param start_idx: start index of the time series
         :param end_idx: end index of the time series
-        :param solver: MIP solver to use
+        :param solver: MIP solver_type to use
         :param batteries_energy_0: initial state of the batteries, if None the default values are taken
         """
         OpfTimeSeries.__init__(self, numerical_circuit=numerical_circuit, start_idx=start_idx, end_idx=end_idx,
-                               solver=solver)
+                               solver=solver, skip_formulation=True)
 
         # build the formulation
         self.problem = self.formulate(batteries_energy_0=batteries_energy_0)
 
     def formulate(self, batteries_energy_0=None):
         """
-        Formulate the AC OPF time series in the non-sequential fashion (all to the solver at once)
+        Formulate the AC OPF time series in the non-sequential fashion (all to the solver_type at once)
         :param batteries_energy_0: initial energy state of the batteries (if none, the default is taken)
         :return: PuLP Problem instance
         """
@@ -331,8 +324,8 @@ class OpfDcTimeSeries(OpfTimeSeries):
         nodal_restrictions = add_dc_nodal_power_balance(self.numerical_circuit, problem, theta, P,
                                                         start_=self.start_idx, end_=self.end_idx)
 
-        load_f, load_t = add_branch_loading_restriction(problem, theta_f, theta_t, Bseries, branch_ratings,
-                                                        branch_rating_slack1, branch_rating_slack2)
+        load_f = add_branch_loading_restriction(problem, theta_f, theta_t, Bseries, branch_ratings,
+                                                branch_rating_slack1, branch_rating_slack2)
 
         # if there are batteries, add the batteries
         if nb > 0:
@@ -347,7 +340,7 @@ class OpfDcTimeSeries(OpfTimeSeries):
         self.E = E.transpose()
         self.load_shedding = load_slack.transpose()
         self.s_from = load_f.transpose()
-        self.s_to = load_t.transpose()
+        self.s_to = -load_f.transpose()
         self.overloads = (branch_rating_slack1 + branch_rating_slack2).transpose()
         self.rating = branch_ratings.T
         self.nodal_restrictions = nodal_restrictions
