@@ -15,10 +15,11 @@
 
 import numpy as np
 from GridCal.Engine.Simulations.result_types import ResultTypes
-from GridCal.Engine.Simulations.results_model import ResultsModel
+from GridCal.Engine.Simulations.results_table import ResultsTable
+from GridCal.Engine.Simulations.results_template import ResultsTemplate
 
 
-class OptimalPowerFlowResults:
+class OptimalPowerFlowResults(ResultsTemplate):
     """
     OPF results.
 
@@ -42,11 +43,37 @@ class OptimalPowerFlowResults:
     """
 
     def __init__(self, bus_names, branch_names, load_names, generator_names, battery_names,
-                 Sbus=None, voltage=None, load_shedding=None, generation_shedding=None,
+                 Sbus=None, voltage=None, load_shedding=None, generator_shedding=None,
                  battery_power=None, controlled_generation_power=None,
                  Sf=None, overloads=None, loading=None, losses=None, converged=None, bus_types=None):
 
-        self.name = 'OPF'
+        ResultsTemplate.__init__(self,
+                                 name='OPF',
+                                 available_results=[ResultTypes.BusVoltageModule,
+                                                    ResultTypes.BusVoltageAngle,
+                                                    ResultTypes.BranchPower,
+                                                    ResultTypes.BranchLoading,
+                                                    ResultTypes.BranchOverloads,
+                                                    ResultTypes.LoadShedding,
+                                                    ResultTypes.ControlledGeneratorShedding,
+                                                    ResultTypes.ControlledGeneratorPower,
+                                                    ResultTypes.BatteryPower],
+                                 data_variables=['bus_names',
+                                                 'branch_names',
+                                                 'load_names',
+                                                 'generator_names',
+                                                 'battery_names',
+                                                 'Sbus',
+                                                 'voltage',
+                                                 'load_shedding',
+                                                 'generator_shedding',
+                                                 'Sf',
+                                                 'bus_types',
+                                                 'overloads',
+                                                 'loading',
+                                                 'battery_power',
+                                                 'generator_power',
+                                                 'converged'])
 
         self.bus_names = bus_names
         self.branch_names = branch_names
@@ -60,8 +87,6 @@ class OptimalPowerFlowResults:
 
         self.load_shedding = load_shedding
 
-        self.generation_shedding = generation_shedding
-
         self.Sf = Sf
 
         self.bus_types = bus_types
@@ -74,24 +99,18 @@ class OptimalPowerFlowResults:
 
         self.battery_power = battery_power
 
-        self.generators_power = controlled_generation_power
+        self.generator_shedding = generator_shedding
 
-        self.flow_direction = None
+        self.generator_power = controlled_generation_power
 
         self.converged = converged
 
-        self.available_results = [ResultTypes.BusVoltageModule,
-                                  ResultTypes.BusVoltageAngle,
-                                  ResultTypes.BranchPower,
-                                  ResultTypes.BranchLoading,
-                                  ResultTypes.BranchOverloads,
-                                  ResultTypes.LoadShedding,
-                                  ResultTypes.ControlledGeneratorShedding,
-                                  ResultTypes.ControlledGeneratorPower,
-                                  ResultTypes.BatteryPower]
-
         self.plot_bars_limit = 100
 
+    def apply_new_rates(self, nc: "SnapshotData"):
+        rates = nc.Rates
+        self.loading = self.Sf / (rates + 1e-9)
+
     def copy(self):
         """
         Return a copy of this
@@ -108,9 +127,9 @@ class OptimalPowerFlowResults:
                                        Sf=self.Sf,
                                        overloads=self.overloads,
                                        loading=self.loading,
-                                       generation_shedding=self.generation_shedding,
+                                       generator_shedding=self.generator_shedding,
                                        battery_power=self.battery_power,
-                                       controlled_generation_power=self.generators_power,
+                                       controlled_generation_power=self.generator_power,
                                        converged=self.converged)
 
     def initialize(self, n, m):
@@ -138,7 +157,7 @@ class OptimalPowerFlowResults:
 
         self.plot_bars_limit = 100
 
-    def mdl(self, result_type) -> "ResultsModel":
+    def mdl(self, result_type) -> "ResultsTable":
         """
         Plot the results
         :param result_type: type of results (string)
@@ -171,7 +190,7 @@ class OptimalPowerFlowResults:
 
         elif result_type == ResultTypes.BranchLoading:
             labels = self.branch_names
-            y = np.abs(self.loading * 100.0)
+            y = self.loading * 100.0
             y_label = '(%)'
             title = 'Branch loading'
 
@@ -195,13 +214,13 @@ class OptimalPowerFlowResults:
 
         elif result_type == ResultTypes.ControlledGeneratorShedding:
             labels = self.generator_names
-            y = self.generation_shedding
+            y = self.generator_shedding
             y_label = '(MW)'
             title = 'Controlled generator shedding'
 
         elif result_type == ResultTypes.ControlledGeneratorPower:
             labels = self.generator_names
-            y = self.generators_power
+            y = self.generator_power
             y_label = '(MW)'
             title = 'Controlled generators power'
 
@@ -217,7 +236,7 @@ class OptimalPowerFlowResults:
             y_label = '(MW)'
             title = 'Battery power'
 
-        mdl = ResultsModel(data=y,
+        mdl = ResultsTable(data=y,
                            index=labels,
                            columns=[result_type.value[0]],
                            title=title,
