@@ -14,12 +14,14 @@
 # along with GridCal.  If not, see <http://www.gnu.org/licenses/>.
 
 import numpy as np
+import pandas as pd
 from GridCal.Engine.Simulations.OPF.opf_driver import OptimalPowerFlowResults
-from GridCal.Engine.Simulations.results_model import ResultsModel
+from GridCal.Engine.Simulations.results_table import ResultsTable
 from GridCal.Engine.Simulations.result_types import ResultTypes
+from GridCal.Engine.Simulations.results_template import ResultsTemplate
 
 
-class OptimalPowerFlowTimeSeriesResults:
+class OptimalPowerFlowTimeSeriesResults(ResultsTemplate):
 
     def __init__(self, bus_names, branch_names, load_names, generator_names, battery_names,
                  n, m, nt, ngen=0, nbat=0, nload=0, time=None, bus_types=()):
@@ -33,7 +35,39 @@ class OptimalPowerFlowTimeSeriesResults:
         :param nload:
         :param time: Time array (optional)
         """
-        self.name = 'OPF time series'
+        ResultsTemplate.__init__(self,
+                                 name='OPF time series',
+                                 available_results=[ResultTypes.BusVoltageModule,
+                                                    ResultTypes.BusVoltageAngle,
+                                                    ResultTypes.ShadowPrices,
+                                                    ResultTypes.BranchPower,
+                                                    ResultTypes.BranchLoading,
+                                                    ResultTypes.BranchOverloads,
+                                                    ResultTypes.LoadShedding,
+                                                    ResultTypes.ControlledGeneratorShedding,
+                                                    ResultTypes.ControlledGeneratorPower,
+                                                    ResultTypes.BatteryPower,
+                                                    ResultTypes.BatteryEnergy],
+                                 data_variables=['bus_names',
+                                                 'branch_names',
+                                                 'load_names',
+                                                 'generator_names',
+                                                 'battery_names',
+                                                 'bus_types',
+                                                 'time',
+                                                 'Sbus',
+                                                 'voltage',
+                                                 'load_shedding',
+                                                 'Sf',
+                                                 'overloads',
+                                                 'loading',
+                                                 'losses',
+                                                 'battery_power',
+                                                 'battery_energy',
+                                                 'generator_shedding',
+                                                 'generator_power',
+                                                 'shadow_prices',
+                                                 'converged'])
 
         self.bus_names = bus_names
         self.branch_names = branch_names
@@ -67,20 +101,6 @@ class OptimalPowerFlowTimeSeriesResults:
 
         self.Sf = np.zeros((nt, m), dtype=complex)
 
-        self.bus_types = np.zeros(n, dtype=int)
-
-        self.available_results = [ResultTypes.BusVoltageModule,
-                                  ResultTypes.BusVoltageAngle,
-                                  ResultTypes.ShadowPrices,
-                                  ResultTypes.BranchPower,
-                                  ResultTypes.BranchLoading,
-                                  ResultTypes.BranchOverloads,
-                                  ResultTypes.LoadShedding,
-                                  ResultTypes.ControlledGeneratorShedding,
-                                  ResultTypes.ControlledGeneratorPower,
-                                  ResultTypes.BatteryPower,
-                                  ResultTypes.BatteryEnergy]
-
         self.generator_power = np.zeros((nt, ngen), dtype=float)
 
         self.generator_shedding = np.zeros((nt, ngen), dtype=float)
@@ -91,6 +111,10 @@ class OptimalPowerFlowTimeSeriesResults:
 
         self.converged = np.empty(nt, dtype=bool)
 
+    def apply_new_time_series_rates(self, nc: "TimeCircuit"):
+        rates = nc.Rates.T
+        self.loading = self.Sf / (rates + 1e-9)
+
     def init_object_results(self, ngen, nbat):
         """
         declare the generator results. This is done separately since these results are known at the end of the simulation
@@ -122,7 +146,7 @@ class OptimalPowerFlowTimeSeriesResults:
 
         self.Sf[t, :] = res.Sf
 
-    def mdl(self, result_type) -> "ResultsModel":
+    def mdl(self, result_type) -> "ResultsTable":
         """
         Plot the results
         :param result_type:
@@ -214,10 +238,10 @@ class OptimalPowerFlowTimeSeriesResults:
             y = np.zeros(0)
 
         if self.time is not None:
-            index = self.time
+            index = pd.to_datetime(self.time)
         else:
             index = np.arange(0, y.shape[0], 1)
 
-        mdl = ResultsModel(data=y, index=index, columns=labels, title=title,
+        mdl = ResultsTable(data=y, index=index, columns=labels, title=title,
                            ylabel=y_label, xlabel='', units=y_label)
         return mdl
