@@ -27,8 +27,6 @@ from GridCal.Engine.Simulations.PowerFlow.power_flow_results import NumericPower
 from GridCal.Engine.Simulations.PowerFlow.discrete_controls import control_q_inside_method
 from GridCal.Engine.Simulations.PowerFlow.high_speed_fubm_jacobian import fubm_jacobian
 from GridCal.Engine.basic_structures import ReactivePowerControlMode
-import GridCal.Engine.Simulations.PowerFlow.derivatives as deriv
-from GridCal.Engine.Sparse.csc import sp_slice, csc_stack_2d_ff, sp_slice_rows
 
 
 @nb.jit(nopython=True, cache=True)
@@ -68,226 +66,6 @@ def compute_converter_losses(V, It, F, alpha1, alpha2, alpha3, iVscL):
     return Gsw
 
 
-def fubm_jacobian__(nb, nl, iPfsh, iPfdp, iQfma, iQtma, iVtma, iBeqz, iBeqv, VfBeqbus, Vtmabus,
-                  F, T, Ys, k2, tap, ma, Bc, Beq, Kdp, V, Ybus, Yf, Yt, Cf, Ct, pvpq, pq):
-    """
-    Compute the FUBM jacobian in a dynamic fashion by only computing the derivatives that are needed
-    :param nb: number of buses
-    :param nl: Number of lines
-    :param iPfsh: indices of the Pf controlled branches
-    :param iPfdp: indices of the droop controlled branches
-    :param iQfma: indices of the Qf controlled branches
-    :param iQtma: Indices of the Qt controlled branches
-    :param iVtma: Indices of the Vt controlled branches
-    :param iBeqz: Indices of the Qf controlled branches
-    :param iBeqv: Indices of the Vf Controlled branches
-    :param F: Array of "from" bus indices
-    :param T: Array of "to" bus indices
-    :param Ys: Array of branch series admittances
-    :param k2: Array of branch converter losses
-    :param tap: Array of complex tap values {remember tap = ma * exp(1j * theta) }
-    :param ma: Array of tap modules
-    :param Bc: Array of branch full susceptances
-    :param Beq: Array of brach equivalent (variable) susceptances
-    :param Kdp: Array of branch converter droop constants
-    :param V: Array of complex bus voltages
-    :param Ybus: Admittance matrix
-    :param Yf: Admittances matrix of the branches with the "from" buses
-    :param Yt: Admittances matrix of the branches with the "to" buses
-    :param Cf: Connectivity matrix of the branches with the "from" buses
-    :param Ct: Connectivity matrix of the branches with the "to" buses
-    :param pvpq: Array of pv and then pq bus indices (not sorted)
-    :param pq: Array of PQ bus indices
-    :return: FUBM Jacobian matrix
-    """
-    nPfsh = len(iPfsh)
-    nPfdp = len(iPfdp)
-    nQfma = len(iQfma)
-    nQtma = len(iQtma)
-    nVtma = len(iVtma)
-    nBeqz = len(iBeqz)
-    nBeqv = len(iBeqv)
-    nVfBeqbus = len(VfBeqbus)
-    nVtmabus = len(Vtmabus)
-    npq = len(pq)
-
-    i2 = np.r_[pq, VfBeqbus, Vtmabus]
-    i4 = np.r_[iQfma, iBeqz]
-
-    # compose the derivatives of the power injections w.r.t Va and Vm
-    dSbus_dVa, dSbus_dVm = deriv.dSbus_dV_csc(Ybus, V)
-
-    # compose the derivatives of the branch flow w.r.t Va and Vm
-    Vc = np.conj(V)
-    E = V / np.abs(V)
-    dSf_dVa, dSf_dVm = deriv.dSf_dV_fast(Yf, V, Vc, E, F, Cf)
-
-    if nQtma:
-        dSt_dVa, dSt_dVm = deriv.dSf_dV_fast(Yt, V, Vc, E, T, Ct)
-
-    # compose the number of columns and rows of the jacobian super structure "mats"
-    cols = 0
-    rows = 0
-
-    # column 1: derivatives w.r.t Va
-    cols += 1
-    j11 = sp_slice(dSbus_dVa.real, pvpq, pvpq)
-    mats = [j11]
-    rows += 1
-
-    if npq + nVfBeqbus + nVtmabus:
-        j21 = sp_slice(dSbus_dVa.imag, i2, pvpq)
-        mats.append(j21)
-        rows += 1
-
-    if nPfsh:
-        j31 = sp_slice(dSf_dVa.real, iPfsh, pvpq)
-        mats.append(j31)
-        rows += 1
-
-    if nQfma + nBeqz:
-        j41 = sp_slice(dSf_dVa.imag, i4, pvpq)
-        mats.append(j41)
-        rows += 1
-
-    if nQtma:
-        j51 = sp_slice(dSt_dVa.imag, iQtma, pvpq)
-        mats.append(j51)
-        rows += 1
-
-    if nPfdp:
-        dPfdp_dVa = -dSf_dVa.real
-        j61 = sp_slice(dPfdp_dVa, iPfdp, pvpq)
-        mats.append(j61)
-        rows += 1
-
-    # column 2: derivatives w.r.t Vm
-    cols += 1
-    j12 = sp_slice(dSbus_dVm.real, pvpq, pq)
-    mats.append(j12)
-
-    if npq + nVfBeqbus + nVtmabus:
-        j22 = sp_slice(dSbus_dVm.imag, i2, pq)
-        mats.append(j22)
-
-    if nPfsh:
-        j32 = sp_slice(dSf_dVm.real, iPfsh, pq)
-        mats.append(j32)
-
-    if nQfma + nBeqz:
-        j42 = sp_slice(dSf_dVm.imag, i4, pq)
-        mats.append(j42)
-
-    if nQtma:
-        j52 = sp_slice(dSt_dVm.imag, iQtma, pq)
-        mats.append(j52)
-
-    if nPfdp:
-        dVmf_dVm = lil_matrix((nl, nb))
-        dVmf_dVm[iPfdp, :] = Cf[iPfdp, :]
-        dPfdp_dVm = -dSf_dVm.real + diags(Kdp) * dVmf_dVm
-        j62 = sp_slice(dPfdp_dVm, iPfdp, pq)
-        mats.append(j62)
-
-    # Column 3: derivatives w.r.t Beq for iBeqz + iBeqv
-    if nBeqz + nBeqv:
-        cols += 1
-        dSbus_dBeqz, dSf_dBeqz, dSt_dBeqz = deriv.derivatives_Beq_csc_fast(nb, nl, np.r_[iBeqz, iBeqv],
-                                                                           F, T, V, ma, k2)
-
-        j13 = sp_slice_rows(dSbus_dBeqz.real, pvpq)
-        mats.append(j13)
-
-        if npq + nVfBeqbus + nVtmabus:
-            j23 = sp_slice_rows(dSbus_dBeqz.imag, i2)
-            mats.append(j23)
-
-        if nPfsh:
-            j33 = sp_slice_rows(dSf_dBeqz.real, iPfsh)
-            mats.append(j33)
-
-        if nQfma + nBeqz:
-            j43 = sp_slice_rows(dSf_dBeqz.imag, i4)
-            mats.append(j43)
-
-        if nQtma:
-            j53 = sp_slice_rows(dSt_dBeqz.imag, iQtma)
-            mats.append(j53)
-
-        if nPfdp:
-            dPfdp_dBeqz = -dSf_dBeqz.real
-            j63 = sp_slice_rows(dPfdp_dBeqz, iPfdp)
-            mats.append(j63)
-
-    # Column 4: derivative w.r.t ma for iQfma + iQfma + iVtma
-    if nQfma + nQtma + nVtma:
-        cols += 1
-        dSbus_dQfma, dSf_dQfma, dSt_dQfma = deriv.derivatives_ma_csc_fast(nb, nl, np.r_[iQfma, iQtma, iVtma],
-                                                                          F, T, Ys, k2, tap, ma, Bc, Beq, V)
-
-        j14 = sp_slice_rows(dSbus_dQfma.real, pvpq)
-        mats.append(j14)
-
-        if npq + nVfBeqbus + nVtmabus:
-            j24 = sp_slice_rows(dSbus_dQfma.imag, i2)
-            mats.append(j24)
-
-        if nPfsh:
-            j34 = sp_slice_rows(dSf_dQfma.real, iPfsh)
-            mats.append(j34)
-
-        if nQfma + nBeqz:
-            j44 = sp_slice_rows(dSf_dQfma.imag, i4)
-            mats.append(j44)
-
-        if nQtma:
-            j54 = sp_slice_rows(dSt_dQfma.imag, iQtma)
-            mats.append(j54)
-
-        if nPfdp:
-            dPfdp_dQfma = -dSf_dQfma.real
-            j64 = sp_slice_rows(dPfdp_dQfma, iPfdp)
-            mats.append(j64)
-
-    # Column 5: derivatives w.r.t theta sh for iPfsh + droop
-    if nPfsh + nPfdp > 0:
-        cols += 1
-        dSbus_dPfx, dSf_dPfx, dSt_dPfx = deriv.derivatives_sh_csc_fast(nb, nl, np.r_[iPfsh, iPfdp],
-                                                                       F, T, Ys, k2, tap, V)
-
-        j15 = sp_slice_rows(dSbus_dPfx.real, pvpq)
-        mats.append(j15)
-
-        if npq + nVfBeqbus + nVtmabus:
-            j25 = sp_slice_rows(dSbus_dPfx.imag, i2)
-            mats.append(j25)
-
-        if nPfsh:
-            j35 = sp_slice_rows(dSf_dPfx.real, iPfsh)
-            mats.append(j35)
-
-        if nQfma + nBeqz:
-            j45 = sp_slice_rows(dSf_dPfx.imag, i4)
-            mats.append(j45)
-
-        if nQtma:
-            j55 = sp_slice_rows(dSt_dPfx.imag, iQtma)
-            mats.append(j55)
-
-        if nPfdp:
-            dPfdp_dPfsh = -dSf_dPfx.real
-            j65 = sp_slice_rows(dPfdp_dPfsh, iPfdp)
-            mats.append(j65)
-
-    # compose Jacobian from the submatrices
-    J = csc_stack_2d_ff(mats, rows, cols, row_major=False)
-
-    if J.shape[0] != J.shape[1]:
-        raise Exception('Invalid Jacobian shape!')
-
-    return J
-
-
 def compute_fx(Ybus, V, Vm, Sbus, Sf, St, Pfset, Qfset, Qtset, Vmfset, Kdp, F,
                pvpq, pq, iPfsh, iQfma, iBeqz, iQtma, iPfdp, VfBeqbus, Vtmabus):
     """
@@ -473,7 +251,7 @@ def NR_LS_ACDC(nc: "SnapshotData", Vbus, Sbus,
                                        mf=nc.branch_data.tap_f,
                                        mt=nc.branch_data.tap_t)
 
-    #  compute branch power flows
+    #  compute branch power Sf
     If = Yf * V  # complex current injected at "from" bus, Yf(br, :) * V; For in-service branches
     It = Yt * V  # complex current injected at "to"   bus, Yt(br, :) * V; For in-service branches
     Sf = V[F] * np.conj(If)  # complex power injected at "from" bus
@@ -573,7 +351,7 @@ def NR_LS_ACDC(nc: "SnapshotData", Vbus, Sbus,
                                                mf=nc.branch_data.tap_f,
                                                mt=nc.branch_data.tap_t)
 
-            #  compute branch power flows
+            #  compute branch power Sf
             If = Yf * V  # complex current injected at "from" bus
             It = Yt * V  # complex current injected at "to" bus
             Sf = V[F] * np.conj(If)  # complex power injected at "from" bus
@@ -627,12 +405,12 @@ def NR_LS_ACDC(nc: "SnapshotData", Vbus, Sbus,
             end = time.time()
             elapsed = end - start
 
-            # set the state for the next solver
+            # set the state for the next solver_type
             nc.branch_data.m[:, 0] = m
             nc.branch_data.theta[:, 0] = theta
             nc.branch_data.Beq[:, 0] = Beq
 
-            return NumericPowerFlowResults(V, converged, norm_f_new, prev_Scalc, m, theta, Beq, iterations, elapsed)
+            return NumericPowerFlowResults(V, converged, norm_f_new, prev_Scalc, m, theta, Beq, Ybus, Yf, Yt, iterations, elapsed)
         else:
             # the iteration was ok, check the controls if the error is small enough
             if norm_f < 1e-2:
@@ -718,7 +496,7 @@ def NR_LS_ACDC(nc: "SnapshotData", Vbus, Sbus,
     end = time.time()
     elapsed = end - start
 
-    return NumericPowerFlowResults(V, converged, norm_f, Scalc, m, theta, Beq, iterations, elapsed)
+    return NumericPowerFlowResults(V, converged, norm_f, Scalc, m, theta, Beq, Ybus, Yf, Yt, iterations, elapsed)
 
 
 def LM_ACDC(nc: "SnapshotData", Vbus, Sbus,
@@ -799,7 +577,7 @@ def LM_ACDC(nc: "SnapshotData", Vbus, Sbus,
                                            mf=nc.branch_data.tap_f,
                                            mt=nc.branch_data.tap_t)
 
-        #  compute branch power flows
+        #  compute branch power Sf
         If = Yf * V  # complex current injected at "from" bus, Yf(br, :) * V; For in-service branches
         It = Yt * V  # complex current injected at "to"   bus, Yt(br, :) * V; For in-service branches
         Sf = V[F] * np.conj(If)  # complex power injected at "from" bus
@@ -929,7 +707,7 @@ def LM_ACDC(nc: "SnapshotData", Vbus, Sbus,
                                                mf=nc.branch_data.tap_f,
                                                mt=nc.branch_data.tap_t)
 
-            #  compute branch power flows
+            #  compute branch power Sf
             If = Yf * V  # complex current injected at "from" bus, Yf(br, :) * V; For in-service branches
             It = Yt * V  # complex current injected at "to"   bus, Yt(br, :) * V; For in-service branches
             Sf = V[F] * np.conj(If)  # complex power injected at "from" bus
@@ -984,11 +762,14 @@ def LM_ACDC(nc: "SnapshotData", Vbus, Sbus,
         converged = True
         Scalc = S0  # V * np.conj(Ybus * V - Ibus)
         iter_ = 0
+        Ybus = None
+        Yf = None
+        Yt = None
 
     end = time.time()
     elapsed = end - start
 
-    return NumericPowerFlowResults(V, converged, norm_f, Scalc, m, theta, Beq, iter_, elapsed)
+    return NumericPowerFlowResults(V, converged, norm_f, Scalc, m, theta, Beq, Ybus, Yf, Yt, iter_, elapsed)
 
 
 if __name__ == "__main__":
