@@ -17,7 +17,8 @@ from enum import Enum
 
 
 class SimulationTypes(Enum):
-    PowerFlow_run = 'power flow'
+    TemplateDriver = 'Template'
+    PowerFlow_run = 'Power flow'
     ShortCircuit_run = 'Short circuit'
     MonteCarlo_run = 'Monte Carlo'
     TimeSeries_run = 'Time series power flow'
@@ -27,11 +28,29 @@ class SimulationTypes(Enum):
     StochasticPowerFlow = 'Stochastic Power Flow'
     Cascade_run = 'Cascade'
     OPF_run = 'Optimal power flow'
-    OPFTimeSeries_run = 'OPF Time series'
+    OPF_NTC_run = 'Optimal net transfer capacity'
+    OPFTimeSeries_run = 'Optimal power flow time series'
     TransientStability_run = 'Transient stability'
     TopologyReduction_run = 'Topology reduction'
-    PTDF_run = 'PTDF'
-    PTDF_TS_run = 'PTDF_TS'
-    OTDF_run = 'OTDF'
-    OTDF_TS_run = 'OTDF_TS'
+    LinearAnalysis_run = 'Linear analysis'
+    LinearAnalysis_TS_run = 'Linear analysis time series'
+    ContingencyAnalysis_run = 'Contingency analysis'
+    ContingencyAnalysisTS_run = 'Contingency analysis time series'
     Delete_and_reduce_run = 'Delete and reduce'
+    NetTransferCapacity_run = 'Available transfer capacity'
+    NetTransferCapacityTS_run = 'Available transfer capacity time series'
+    SigmaAnalysis_run = "Sigma Analysis"
+    NodeGrouping_run = "Node groups"
+
+    def __str__(self):
+        return self.value
+
+    def __repr__(self):
+        return str(self)
+
+    @staticmethod
+    def argparse(s):
+        try:
+            return SimulationTypes[s]
+        except KeyError:
+            return s
