@@ -78,6 +78,12 @@ class ResultTypes(Enum):
     ControlledGeneratorShedding = 'Generator shedding', DeviceType.GeneratorDevice
     ControlledGeneratorPower = 'Generator power', DeviceType.GeneratorDevice
 
+    # OPF-NTC
+    HvdcOverloads = 'HVDC overloads', DeviceType.HVDCLineDevice
+    NodeSlacks = 'Nodal slacks', DeviceType.BusDevice
+    GenerationDelta = 'Generation deltas', DeviceType.GeneratorDevice
+    InterAreaExchange = 'Inter-Area exchange', DeviceType.NoDevice
+
     # Short-circuit
     BusShortCircuitPower = 'Bus short circuit power', DeviceType.BusDevice
 
@@ -88,7 +94,7 @@ class ResultTypes(Enum):
     OTDF = 'Outage transfer distribution factors', DeviceType.BranchDevice
 
     MaxOverloads = 'Maximum contingency flow', DeviceType.BranchDevice
-    WorstContingencyFlows = 'Worst contingency flows', DeviceType.BranchDevice
+    WorstContingencyFlows = 'Worst contingency Sf', DeviceType.BranchDevice
     WorstContingencyLoading = 'Worst contingency loading', DeviceType.BranchDevice
     ContingencyFrequency = 'Contingency frequency', DeviceType.BranchDevice
     ContingencyRelativeFrequency = 'Contingency relative frequency', DeviceType.BranchDevice
@@ -103,5 +109,25 @@ class ResultTypes(Enum):
     SigmaDistances = 'Sigma distances', DeviceType.BusDevice
     SigmaPlusDistances = 'Sigma + distances', DeviceType.BusDevice
 
+    # ATC
+    AvailableTransferCapacityMatrix = 'Available transfer capacity', DeviceType.BranchDevice
+    AvailableTransferCapacity = 'Available transfer capacity (final)', DeviceType.BranchDevice
+    AvailableTransferCapacityN = 'Available transfer capacity (N)', DeviceType.BranchDevice
+    AvailableTransferCapacityAlpha = 'Sensitivity to the exchange', DeviceType.BranchDevice
+    AvailableTransferCapacityBeta = 'Sensitivity to the exchange (N-1)', DeviceType.BranchDevice
+    NetTransferCapacity = 'Net transfer capacity', DeviceType.BranchDevice
+    AvailableTransferCapacityReport = 'ATC Report', DeviceType.NoDevice
 
+    def __str__(self):
+        return self.value
+
+    def __repr__(self):
+        return str(self)
+
+    @staticmethod
+    def argparse(s):
+        try:
+            return ResultTypes[s]
+        except KeyError:
+            return s
 
