@@ -12,70 +12,11 @@ import math
 from PySide2.QtWidgets import *
 
 from GridCal.Gui.Analysis.gui import *
+from GridCal.Gui.GuiFunctions import PandasModel, get_list_model
 from GridCal.Engine.Core.multi_circuit import MultiCircuit
 from GridCal.Engine.Devices import *
 
 
-class PandasModel(QtCore.QAbstractTableModel):
-    """
-    Class to populate a Qt table view with a pandas data frame
-    """
-    def __init__(self, data, parent=None):
-        QtCore.QAbstractTableModel.__init__(self, parent)
-        self._data = np.array(data.values)
-        self._cols = data.columns
-        self.index = data.index.values
-        self.r, self.c = np.shape(self._data)
-        self.isDate = False
-        if isinstance(self.index[0], np.datetime64):
-            self.index = pd.to_datetime(self.index)
-            self.isDate = True
-
-        self.formatter = lambda x: "%.2f" % x
-
-    def rowCount(self, parent=None):
-        return self.r
-
-    def columnCount(self, parent=None):
-        return self.c
-
-    def data(self, index, role=QtCore.Qt.DisplayRole):
-        if index.isValid():
-            if role == QtCore.Qt.DisplayRole:
-                return str(self._data[index.row(), index.column()])
-        return None
-
-    def headerData(self, p_int, orientation, role):
-        if role == QtCore.Qt.DisplayRole:
-            if orientation == QtCore.Qt.Horizontal:
-                return self._cols[p_int]
-            elif orientation == QtCore.Qt.Vertical:
-                if self.index is None:
-                    return p_int
-                else:
-                    if self.isDate:
-                        return self.index[p_int].strftime('%Y/%m/%d  %H:%M.%S')
-                    else:
-                        return str(self.index[p_int])
-        return None
-
-
-def get_list_model(iterable):
-    """
-    get Qt list model from a simple iterable
-    :param iterable: 
-    :return: List model
-    """
-    list_model = QtGui.QStandardItemModel()
-    if iterable is not None:
-        for val in iterable:
-            # for the list model
-            item = QtGui.QStandardItem(val)
-            item.setEditable(False)
-            list_model.appendRow(item)
-    return list_model
-
-
 class GridErrorLog:
 
     def __init__(self, parent=None):
@@ -355,7 +296,7 @@ class GridAnalysisGUI(QtWidgets.QMainWindow):
         else:
             self.msg('Select a data structure')
 
-    def analyze_all(self, imbalance_threshold=0.1, v_low=0.95, v_high=1.05):
+    def analyze_all(self, imbalance_threshold=0.02, v_low=0.95, v_high=1.05):
         """
         Analyze the model data
         :param imbalance_threshold: Allowed percentage of imbalance
@@ -496,10 +437,10 @@ class GridAnalysisGUI(QtWidgets.QMainWindow):
                 elements = self.circuit.get_generators()
 
                 for k, obj in enumerate(elements):
-                    Pg += obj.P
+                    Pg += obj.P * obj.active
 
                     if self.circuit.time_profile is not None:
-                        Pg_prof += obj.P_prof
+                        Pg_prof += obj.P_prof * obj.active_prof
 
                     if obj.Vset < v_low:
                         self.log.add(object_type='Generator',
@@ -520,21 +461,21 @@ class GridAnalysisGUI(QtWidgets.QMainWindow):
                 elements = self.circuit.get_batteries()
 
                 for obj in elements:
-                    Pg += obj.P
+                    Pg += obj.P * obj.active
 
                     if self.circuit.time_profile is not None:
-                        Pg_prof += obj.P_prof
+                        Pg_prof += obj.P_prof * obj.active_prof
 
             elif object_type == DeviceType.StaticGeneratorDevice.value:
                 elements = self.circuit.get_static_generators()
 
                 for k, obj in enumerate(elements):
-                    Pg += obj.P
-                    Qg += obj.Q
+                    Pg += obj.P * obj.active
+                    Qg += obj.Q * obj.active
 
                     if self.circuit.time_profile is not None:
-                        Pg_prof += obj.P_prof
-                        Qg_prof += obj.Q_prof
+                        Pg_prof += obj.P_prof * obj.active_prof
+                        Qg_prof += obj.Q_prof * obj.active_prof
 
             elif object_type == DeviceType.ShuntDevice.value:
                 elements = self.circuit.get_shunts()
@@ -543,12 +484,12 @@ class GridAnalysisGUI(QtWidgets.QMainWindow):
                 elements = self.circuit.get_loads()
 
                 for obj in elements:
-                    Pl += obj.P
-                    Ql += obj.Q
+                    Pl += obj.P * obj.active
+                    Ql += obj.Q * obj.active
 
                     if self.circuit.time_profile is not None:
-                        Pl_prof += obj.P_prof
-                        Ql_prof += obj.Q_prof
+                        Pl_prof += obj.P_prof * obj.active_prof
+                        Ql_prof += obj.Q_prof * obj.active_prof
 
         # compare loads
         p_ratio = abs(Pl - Pg) / (Pl + 1e-20)
@@ -580,21 +521,21 @@ class GridAnalysisGUI(QtWidgets.QMainWindow):
                 p_ratio = abs(Pl_prof[t] - Pg_prof[t]) / (Pl_prof[t] + 1e-20)
                 if p_ratio > imbalance_threshold:
                     msg = "{:.1f}".format(p_ratio * 100) + "% >> " + str(imbalance_threshold) + "%"
-                    self.log.add(object_type='Grid time events',
+                    self.log.add(object_type='Active power balance',
                                  element_name=self.circuit,
                                  element_index=t,
                                  severity='High',
-                                 propty='Active power balance ' + msg,
+                                 propty=msg,
                                  message='There is too much active power imbalance')
 
                 # compare reactive power limits
                 if not (Qmin <= -Ql_prof[t] <= Qmax):
                     msg = "Reactive power out of bounds {0}<={1}<={2}".format(Qmin, Ql, Qmax)
-                    self.log.add(object_type='Grid time events',
+                    self.log.add(object_type='Reactive power power balance',
                                  element_name=self.circuit,
                                  element_index=t,
                                  severity='High',
-                                 propty='Reactive power power balance ' + msg,
+                                 propty=msg,
                                  message='There is too much reactive power imbalance')
 
         # set logs
