@@ -39,6 +39,7 @@ from GridCal.Gui.GridEditorWidget.transformer2w_graphics import TransformerGraph
 from GridCal.Gui.GridEditorWidget.hvdc_graphics import HvdcGraphicItem
 from GridCal.Gui.GridEditorWidget.vsc_graphics import VscGraphicItem
 from GridCal.Gui.GridEditorWidget.upfc_graphics import UpfcGraphicItem
+from GridCal.Engine.Simulations.driver_types import SimulationTypes
 from matplotlib import pyplot as plt
 
 '''
@@ -215,6 +216,10 @@ class DiagramScene(QGraphicsScene):
         super(DiagramScene, self).__init__(parent)
         self.parent_ = parent
         self.circuit = circuit
+        self.results_dictionary = dict()
+
+    def set_results_to_plot(self, all_threads):
+        self.results_dictionary = {thr.tpe: thr for thr in all_threads if thr is not None}
 
     def plot_bus(self, i, api_object):
         """
@@ -237,10 +242,10 @@ class DiagramScene(QGraphicsScene):
                 power_data = api_object.get_active_injection_profiles_dictionary()
                 voltage = dict()
 
-                for key, driver in self.circuit.results_dictionary.items():
+                for key, driver in self.results_dictionary.items():
                     if hasattr(driver, 'results'):
                         if driver.results is not None:
-                            if key == 'Time Series':
+                            if key == SimulationTypes.TimeSeries_run:
                                 voltage[key] = np.abs(driver.results.voltage[:, i])
 
                 # injections
@@ -292,29 +297,42 @@ class DiagramScene(QGraphicsScene):
                 loading_clustering_data = None
                 power_clustering_data = None
 
-                for key, driver in self.circuit.results_dictionary.items():
+                for key, driver in self.results_dictionary.items():
                     if hasattr(driver, 'results'):
                         if driver.results is not None:
-                            if key == 'Time Series':
-                                power_data[key] = driver.results.Sf.real[:, i]
-                                loading_data[key] = np.sort(np.abs(driver.results.loading.real[:, i] * 100.0))
+                            if key == SimulationTypes.TimeSeries_run:
+                                power_data[key.value] = driver.results.Sf.real[:, i]
+                                loading_data[key.value] = np.sort(np.abs(driver.results.loading.real[:, i] * 100.0))
 
-                            elif key == 'Time Series Clustering':
+                            elif key == SimulationTypes.ClusteringTimeSeries_run:
                                 x_cl = x[driver.sampled_time_idx]
                                 power_clustering_data = driver.results.Sf.real[:, i]
                                 loading_clustering_data = np.sort(np.abs(driver.results.loading.real[:, i] * 100.0))
 
-                            elif key == 'PTDF Time Series':
-                                power_data[key] = driver.results.Sf.real[:, i]
-                                loading_data[key] = np.sort(np.abs(driver.results.loading.real[:, i] * 100.0))
+                            elif key == SimulationTypes.LinearAnalysis_TS_run:
+                                power_data[key.value] = driver.results.Sf.real[:, i]
+                                loading_data[key.value] = np.sort(np.abs(driver.results.loading.real[:, i] * 100.0))
 
-                            elif key == 'N-1 time series':
-                                power_data[key] = driver.results.worst_flows.real[:, i]
-                                loading_data[key] = np.sort(np.abs(driver.results.worst_loading.real[:, i] * 100.0))
+                            elif key == SimulationTypes.NetTransferCapacityTS_run:
+                                power_data[key.value] = driver.results.atc[:, i]
+                                atc_perc = driver.results.atc[:, i] / (api_object.rate_prof + 1e-9)
+                                loading_data[key.value] = np.sort(np.abs(atc_perc * 100.0))
 
-                            elif key == 'Stochastic Power Flow':
+                            elif key == SimulationTypes.ContingencyAnalysisTS_run:
+                                power_data[key.value] = driver.results.worst_flows.real[:, i]
+                                loading_data[key.value] = np.sort(np.abs(driver.results.worst_loading.real[:, i] * 100.0))
+
+                            elif key == SimulationTypes.OPFTimeSeries_run:
+                                power_data[key.value] = driver.results.Sf.real[:, i]
+                                loading_data[key.value] = np.sort(np.abs(driver.results.loading.real[:, i] * 100.0))
+
+                            elif key == SimulationTypes.StochasticPowerFlow:
                                 loading_st_data = np.sort(np.abs(driver.results.loading_points.real[:, i] * 100.0))
 
+                # add the rating
+                # power_data['Rates+'] = api_object.rate_prof
+                # power_data['Rates-'] = -api_object.rate_prof
+
                 # loading
                 if len(loading_data.keys()):
                     df = pd.DataFrame(data=loading_data, index=p)
@@ -324,14 +342,18 @@ class DiagramScene(QGraphicsScene):
 
                 if loading_clustering_data is not None:
                     p_st = np.arange(len(loading_clustering_data)).astype(float) / len(loading_clustering_data)
-                    df = pd.DataFrame(data=loading_clustering_data, index=p_st, columns=['Clustering Time Series'])
+                    df = pd.DataFrame(data=loading_clustering_data,
+                                      index=p_st,
+                                      columns=[SimulationTypes.ClusteringTimeSeries_run.value])
                     ax_1.set_title('Probability x < value', fontsize=14)
                     ax_1.set_ylabel('Loading [%]', fontsize=11)
                     df.plot(ax=ax_1)
 
                 if loading_st_data is not None:
                     p_st = np.arange(len(loading_st_data)).astype(float) / len(loading_st_data)
-                    df = pd.DataFrame(data=loading_st_data, index=p_st, columns=['Stochastic Power Flow'])
+                    df = pd.DataFrame(data=loading_st_data,
+                                      index=p_st,
+                                      columns=[SimulationTypes.StochasticPowerFlow.value])
                     ax_1.set_title('Probability x < value', fontsize=14)
                     ax_1.set_ylabel('Loading [%]', fontsize=11)
                     df.plot(ax=ax_1)
@@ -342,9 +364,13 @@ class DiagramScene(QGraphicsScene):
                     ax_2.set_title('Power', fontsize=14)
                     ax_2.set_ylabel('Power [MW]', fontsize=11)
                     df.plot(ax=ax_2)
+                    ax_2.plot(x, api_object.rate_prof, c='gray', linestyle='dashed', linewidth=1)
+                    ax_2.plot(x, -api_object.rate_prof, c='gray', linestyle='dashed', linewidth=1)
 
                 if power_clustering_data is not None:
-                    df = pd.DataFrame(data=power_clustering_data, index=x_cl, columns=['Clustering Time Series'])
+                    df = pd.DataFrame(data=power_clustering_data,
+                                      index=x_cl,
+                                      columns=[SimulationTypes.ClusteringTimeSeries_run.value])
                     ax_2.set_title('Power', fontsize=14)
                     ax_2.set_ylabel('Power [MW]', fontsize=11)
                     df.plot(ax=ax_2)
@@ -872,7 +898,7 @@ class GridEditor(QSplitter):
         terminal_from = branch.bus_from.graphic_obj.terminal
         terminal_to = branch.bus_to.graphic_obj.terminal
         graphic_obj = DcLineGraphicItem(terminal_from, terminal_to, self.diagramScene, branch=branch)
-        graphic_obj.diagramScene.circuit = self.circuit  # add pointer to the circuit
+        graphic_obj.diagramScene.grid = self.circuit  # add pointer to the circuit
         terminal_from.hosting_connections.append(graphic_obj)
         terminal_to.hosting_connections.append(graphic_obj)
         graphic_obj.redraw()
@@ -939,7 +965,7 @@ class GridEditor(QSplitter):
 
         graphic_obj = DcLineGraphicItem(terminal_from, terminal_to, self.diagramScene, branch=branch)
 
-        graphic_obj.diagramScene.circuit = self.circuit  # add pointer to the circuit
+        graphic_obj.diagramScene.grid = self.circuit  # add pointer to the circuit
         terminal_from.hosting_connections.append(graphic_obj)
         terminal_to.hosting_connections.append(graphic_obj)
         graphic_obj.redraw()
@@ -956,7 +982,7 @@ class GridEditor(QSplitter):
 
         graphic_obj = HvdcGraphicItem(terminal_from, terminal_to, self.diagramScene, branch=branch)
 
-        graphic_obj.diagramScene.circuit = self.circuit  # add pointer to the circuit
+        graphic_obj.diagramScene.grid = self.circuit  # add pointer to the circuit
         terminal_from.hosting_connections.append(graphic_obj)
         terminal_to.hosting_connections.append(graphic_obj)
         graphic_obj.redraw()
@@ -973,7 +999,7 @@ class GridEditor(QSplitter):
 
         graphic_obj = VscGraphicItem(terminal_from, terminal_to, self.diagramScene, branch=branch)
 
-        graphic_obj.diagramScene.circuit = self.circuit  # add pointer to the circuit
+        graphic_obj.diagramScene.grid = self.circuit  # add pointer to the circuit
         terminal_from.hosting_connections.append(graphic_obj)
         terminal_to.hosting_connections.append(graphic_obj)
         graphic_obj.redraw()
@@ -990,7 +1016,7 @@ class GridEditor(QSplitter):
 
         graphic_obj = UpfcGraphicItem(terminal_from, terminal_to, self.diagramScene, branch=branch)
 
-        graphic_obj.diagramScene.circuit = self.circuit  # add pointer to the circuit
+        graphic_obj.diagramScene.grid = self.circuit  # add pointer to the circuit
         terminal_from.hosting_connections.append(graphic_obj)
         terminal_to.hosting_connections.append(graphic_obj)
         graphic_obj.redraw()
@@ -1020,16 +1046,7 @@ class GridEditor(QSplitter):
         :param line: Line instance
         :return: Nothing
         """
-        hvdc = HvdcLine(bus_from=line.bus_from,
-                        bus_to=line.bus_to,
-                        name='HVDC Line',
-                        active=line.active,
-                        rate=line.rate,
-                        active_prof=line.active_prof,
-                        rate_prof=line.rate_prof)
-
-        # add device to the circuit
-        self.circuit.add_hvdc(hvdc)
+        hvdc = self.circuit.convert_line_to_hvdc(line)
 
         # add device to the schematic
         hvdc.graphic_obj = self.add_api_hvdc(hvdc)
@@ -1038,9 +1055,6 @@ class GridEditor(QSplitter):
         hvdc.graphic_obj.fromPort.update()
         hvdc.graphic_obj.toPort.update()
 
-        # delete the line from the circuit
-        self.circuit.delete_line(line)
-
         # delete from the schematic
         self.diagramScene.removeItem(line.graphic_obj)
 
@@ -1050,19 +1064,7 @@ class GridEditor(QSplitter):
         :param line: Line instance
         :return: Nothing
         """
-        transformer = Transformer2W(bus_from=line.bus_from,
-                                    bus_to=line.bus_to,
-                                    name='Transformer',
-                                    active=line.active,
-                                    rate=line.rate,
-                                    r=line.R,
-                                    x=line.X,
-                                    b=line.B,
-                                    active_prof=line.active_prof,
-                                    rate_prof=line.rate_prof)
-
-        # add device to the circuit
-        self.circuit.add_transformer2w(transformer)
+        transformer = self.circuit.convert_line_to_transformer(line)
 
         # add device to the schematic
         transformer.graphic_obj = self.add_api_transformer(transformer)
@@ -1071,9 +1073,6 @@ class GridEditor(QSplitter):
         transformer.graphic_obj.fromPort.update()
         transformer.graphic_obj.toPort.update()
 
-        # delete the line from the circuit
-        self.circuit.delete_line(line)
-
         # delete from the schematic
         self.diagramScene.removeItem(line.graphic_obj)
 
@@ -1083,20 +1082,7 @@ class GridEditor(QSplitter):
         :param line: Line instance
         :return: Nothing
         """
-        vsc = VSC(bus_from=line.bus_from,
-                  bus_to=line.bus_to,
-                  name='VSC',
-                  active=line.active,
-                  rate=line.rate,
-                  r1=line.R,
-                  x1=line.X,
-                  Beq=line.B,
-                  m=1.0,
-                  active_prof=line.active_prof,
-                  rate_prof=line.rate_prof)
-
-        # add device to the circuit
-        self.circuit.add_vsc(vsc)
+        vsc = self.circuit.convert_line_to_vsc(line)
 
         # add device to the schematic
         vsc.graphic_obj = self.add_api_vsc(vsc)
@@ -1105,9 +1091,6 @@ class GridEditor(QSplitter):
         vsc.graphic_obj.fromPort.update()
         vsc.graphic_obj.toPort.update()
 
-        # delete the line from the circuit
-        self.circuit.delete_line(line)
-
         # delete from the schematic
         self.diagramScene.removeItem(line.graphic_obj)
 
@@ -1117,19 +1100,7 @@ class GridEditor(QSplitter):
         :param line: Line instance
         :return: Nothing
         """
-        upfc = UPFC(bus_from=line.bus_from,
-                    bus_to=line.bus_to,
-                    name='UPFC',
-                    active=line.active,
-                    rate=line.rate,
-                    rl=line.R,
-                    xl=line.X,
-                    bl=line.B,
-                    active_prof=line.active_prof,
-                    rate_prof=line.rate_prof)
-
-        # add device to the circuit
-        self.circuit.add_upfc(upfc)
+        upfc = self.circuit.convert_line_to_upfc(line)
 
         # add device to the schematic
         upfc.graphic_obj = self.add_api_upfc(upfc)
@@ -1138,9 +1109,6 @@ class GridEditor(QSplitter):
         upfc.graphic_obj.fromPort.update()
         upfc.graphic_obj.toPort.update()
 
-        # delete the line from the circuit
-        self.circuit.delete_line(line)
-
         # delete from the schematic
         self.diagramScene.removeItem(line.graphic_obj)
 
@@ -1154,6 +1122,7 @@ class GridEditor(QSplitter):
         :param transformers2w: list of Transformer Objects
         :param hvdc_lines: list of HvdcLine objects
         :param vsc_devices: list Vsc objects
+        :param upfc_devices: List of UPFC devices
         :param explode_factor: factor of "explosion": Separation of the nodes factor
         :param prog_func: progress report function
         :param text_func: Text report function
@@ -1251,6 +1220,9 @@ class GridEditor(QSplitter):
         :param text_func: Text report function
         """
 
+        # reset zoom level, otherwise the newly loaded grids appear with a much wider spacing
+        self.diagramView.resetTransform()
+
         self.add_elements_to_schematic(buses=circuit.buses,
                                        lines=circuit.lines,
                                        dc_lines=circuit.dc_lines,
@@ -1291,6 +1263,9 @@ class GridEditor(QSplitter):
         #  center the view
         self.center_nodes()
 
+    def clear(self):
+        self.diagramView.scene_.clear()
+
     def schematic_from_api(self, explode_factor=1.0, prog_func=None, text_func=None):
         """
         Generate schematic from the API
@@ -1300,7 +1275,7 @@ class GridEditor(QSplitter):
         :return: Nothing
         """
         # clear all
-        self.diagramView.scene_.clear()
+        self.clear()
 
         # add to schematic
         self.add_circuit_to_schematic(self.circuit,
