@@ -178,6 +178,9 @@ class HvdcGraphicItem(QGraphicsLineItem):
             pe.setChecked(self.api_object.active)
             pe.triggered.connect(self.enable_disable_toggle)
 
+            pe2 = menu.addAction('Convert to Multi-terminal')
+            pe2.triggered.connect(self.convert_to_multi_terminal)
+
             ra6 = menu.addAction('Plot profiles')
             plot_icon = QIcon()
             plot_icon.addPixmap(QPixmap(":/Icons/icons/plot.svg"))
@@ -229,15 +232,18 @@ class HvdcGraphicItem(QGraphicsLineItem):
 
         pass
 
-    def remove(self):
+    def remove(self, ask=True):
         """
         Remove this object in the diagram and the API
         @return:
         """
-        ok = yes_no_question('Do you want to remove this HVDC line?', 'Remove HVDC line')
+        if ask:
+            ok = yes_no_question('Do you want to remove this HVDC line?', 'Remove HVDC line')
+        else:
+            ok = True
 
         if ok:
-            self.diagramScene.circuit.delete_branch(self.api_object)
+            self.diagramScene.circuit.delete_hvdc_line(self.api_object)
             self.diagramScene.removeItem(self)
 
     def reduce(self):
@@ -340,6 +346,9 @@ class HvdcGraphicItem(QGraphicsLineItem):
         # Set pen for everyone
         self.set_pen(QPen(self.color, self.width, self.style))
 
+    def convert_to_multi_terminal(self):
+        pass
+
     def plot_profiles(self):
         """
         Plot the time series profiles
