@@ -62,6 +62,9 @@ class LineEditor(QDialog):
         Ybase = 1 / Zbase
         length = self.line.length
 
+        if length == 0:
+            length = 1.0
+
         R = self.line.R * Zbase / length
         X = self.line.X * Zbase / length
         B = self.line.B * Ybase / length
@@ -179,11 +182,11 @@ class LineEditor(QDialog):
         Set the values
         :return:
         """
-        l = self.l_spinner.value()
+        length = self.l_spinner.value()
         I = self.i_spinner.value()
-        R = self.r_spinner.value() * l
-        X = self.x_spinner.value() * l
-        B = self.b_spinner.value() * l
+        R = self.r_spinner.value() * length
+        X = self.x_spinner.value() * length
+        B = self.b_spinner.value() * length
 
         Vf = self.line.bus_from.Vnom
         Vt = self.line.bus_to.Vnom
@@ -195,6 +198,7 @@ class LineEditor(QDialog):
         self.line.X = np.round(X / Zbase, 6)
         self.line.B = np.round(B / Ybase, 6)
         self.line.rate = np.round(I * Vf * 1.73205080757, 6)  # nominal power in MVA = kA * kV
+        self.line.length = length
 
         if self.selected_template is not None:
             self.line.template = self.selected_template
@@ -503,15 +507,18 @@ class LineGraphicItem(QGraphicsLineItem):
             # change state
             self.enable_disable_toggle()
 
-    def remove(self):
+    def remove(self, ask=True):
         """
         Remove this object in the diagram and the API
         @return:
         """
-        ok = yes_no_question('Do you want to remove this line?', 'Remove line')
+        if ask:
+            ok = yes_no_question('Do you want to remove this line?', 'Remove line')
+        else:
+            ok = True
 
         if ok:
-            self.diagramScene.circuit.delete_branch(self.api_object)
+            self.diagramScene.circuit.delete_line(self.api_object)
             self.diagramScene.removeItem(self)
 
     def reduce(self):
@@ -523,7 +530,7 @@ class LineGraphicItem(QGraphicsLineItem):
 
         if ok:
             # get the index of the branch
-            br_idx = self.diagramScene.circuit.branches.index(self.api_object)
+            br_idx = self.diagramScene.circuit.lines.index(self.api_object)
 
             # call the reduction routine
             removed_branch, removed_bus, \
@@ -791,10 +798,13 @@ class LineGraphicItem(QGraphicsLineItem):
         Convert this object to VSC
         :return:
         """
-        ok = yes_no_question('Are you sure that you want to convert this line into a VSC device?', 'Convert line')
-        if ok:
-            editor = self.diagramScene.parent()
-            editor.convert_line_to_vsc(self.api_object)
+        if self.api_object.convertible_to_vsc():
+            ok = yes_no_question('Are you sure that you want to convert this line into a VSC device?', 'Convert line')
+            if ok:
+                editor = self.diagramScene.parent()
+                editor.convert_line_to_vsc(self.api_object)
+        else:
+            warning_msg('Unable to convert to VSC. One of the buses must be DC and the other AC.')
 
     def to_upfc(self):
         """
