@@ -375,12 +375,15 @@ class UpfcGraphicItem(QGraphicsLineItem):
             # change state
             self.enable_disable_toggle()
 
-    def remove(self):
+    def remove(self, ask=True):
         """
         Remove this object in the diagram and the API
         @return:
         """
-        ok = yes_no_question('Do you want to remove this UPFC?', 'Remove UPFC')
+        if ask:
+            ok = yes_no_question('Do you want to remove this UPFC?', 'Remove UPFC')
+        else:
+            ok = True
 
         if ok:
             self.diagramScene.circuit.delete_branch(self.api_object)
