@@ -323,6 +323,18 @@ class VscGraphicItem(QGraphicsLineItem):
             ra6.setIcon(plot_icon)
             ra6.triggered.connect(self.plot_profiles)
 
+            ra4 = menu.addAction('Assign rate to profile')
+            ra4_icon = QIcon()
+            ra4_icon.addPixmap(QPixmap(":/Icons/icons/assign_to_profile.svg"))
+            ra4.setIcon(ra4_icon)
+            ra4.triggered.connect(self.assign_rate_to_profile)
+
+            ra5 = menu.addAction('Assign active state to profile')
+            ra5_icon = QIcon()
+            ra5_icon.addPixmap(QPixmap(":/Icons/icons/assign_to_profile.svg"))
+            ra5.setIcon(ra5_icon)
+            ra5.triggered.connect(self.assign_status_to_profile)
+
             menu.addSeparator()
 
             re = menu.addAction('Reduce')
@@ -363,15 +375,18 @@ class VscGraphicItem(QGraphicsLineItem):
             # change state
             self.enable_disable_toggle()
 
-    def remove(self):
+    def remove(self, ask=True):
         """
         Remove this object in the diagram and the API
         @return:
         """
-        ok = yes_no_question('Do you want to remove this VSC?', 'Remove VSC')
+        if ask:
+            ok = yes_no_question('Do you want to remove this VSC?', 'Remove VSC')
+        else:
+            ok = True
 
         if ok:
-            self.diagramScene.circuit.delete_branch(self.api_object)
+            self.diagramScene.circuit.delete_vsc_converter(self.api_object)
             self.diagramScene.removeItem(self)
 
     def reduce(self):
@@ -473,6 +488,18 @@ class VscGraphicItem(QGraphicsLineItem):
         i = self.diagramScene.circuit.get_branches().index(self.api_object)
         self.diagramScene.plot_branch(i, self.api_object)
 
+    def assign_rate_to_profile(self):
+        """
+        Assign the snapshot rate to the profile
+        """
+        self.diagramScene.set_rate_to_profile(self.api_object)
+
+    def assign_status_to_profile(self):
+        """
+        Assign the snapshot rate to the profile
+        """
+        self.diagramScene.set_active_status_to_profile(self.api_object)
+
     def setFromPort(self, fromPort):
         """
         Set the From terminal in a connection
